@@ -4,33 +4,41 @@
  *                  Fritz Elfert,    <felfert@to.com>
  *                  Carsten Otte,    <cotte@de.ibm.com>
  *                  Volker Sameske,  <sameske@de.ibm.com>
- *                  Holger Smolinski <smolinsk@de.ibm.com>
+ *                  Holger Smolinski,<smolinsk@de.ibm.com>
+ *                  Gerhard Tonn     <ton@de.ibm.com>
  * Bugreports.to..: <Linux390@de.ibm.com>
- * (C) IBM Corporation, IBM Deutschland Entwicklung GmbH, 1999-2002
- *
- * versions:
- *   1.01   2001-09-07    progressbar, hashmarks, new blksize test
- *   1.02   2001-10-01    main structure dasdfmt_info
- *                        signal handler for SIGTERM, SIGINT and SIGQUIT
- *                        cylinder-wise formatting in every case
- *   1.03   2001-10-31    some dasdfmt return codes changed
- *   1.04   2002-02-27    do some checks with volume serial
- *          2002-03-12    removed dependency to kernel headers
- *   1.05   2002-03-18    major code restructuring
- *                        removed dependency to proc-filesystem
- *                        removed unused range formatting stuff
- *   1.06   2002-05-10    changed API handling, set it back to 0
+ * (C) IBM Corporation, IBM Deutschland Entwicklung GmbH, 1999-2004
  */
 
+#include <sys/utsname.h>
+#include <linux/version.h>
+
+#include "zt_common.h"
 #include "dasdfmt.h"
 #include "vtoc.h" 
 
+/* Full tool name */
+static const char tool_name[] = "zSeries DASD format program";
+
+/* Copyright notice */
+static const char copyright_notice[] = "(C) Copyright IBM Corp. 1999-2004";
+
+/*
+ * Print version information.
+ */
+static void
+print_version (void)
+{
+	printf ("%s version %s.\n", tool_name, RELEASE_STRING);
+	printf ("%s.\n", copyright_notice);
+}
+
 /*
  * print the usage text and exit
  */
 static void exit_usage(int exitcode)
 {
-	printf("Usage: %s [-htvyLVF]\n"
+	printf("Usage: %s [-htvyLVFk]\n"
 	       "	       [-l <volser>      | --label=<volser>]\n"
 	       "               [-b <blocksize>   | --blocksize=<blocksize>]\n"
 	       "               [-d <disk layout> | --disk_layout=<disk layout>]\n"
@@ -43,7 +51,8 @@ static void exit_usage(int exitcode)
 	       "       -m x or --hashmarks=x means show a hashmark every x "
 	       "cylinders\n"
 	       "       -v means verbose mode\n"
-	       "       -F means don't check if the device is in use\n\n"
+	       "       -F means don't check if the device is in use\n"
+	       "       -k means keep volume serial\n\n"
 	       "       <volser> is the volume identifier, which is converted\n"
 	       "                to EBCDIC and written to disk. \n"
 	       "                (6 characters, e.g. LNX001\n"
@@ -60,7 +69,9 @@ static void exit_usage(int exitcode)
 	       "           and alternatively\n"
 	       "           -n xxxx or --devno=xxxx\n"
 	       "           in case you are using devfs.\n"
-	       "           xxxx is your hexadecimal device number.\n");
+	       "           xxxx is your hexadecimal device number.\n"
+	       "\n"
+	       "Please report bugs to: linux390@de.ibm.com\n");
 	exit(exitcode);
 }
 
@@ -78,7 +89,7 @@ static void program_interrupt_signal (int sig)
 
 	if (disk_disabled) {
 		printf("Re-accessing the device... \n");
-		rc = ioctl(filedes, BIODASDENABLE, format_params);
+		rc = ioctl(filedes, BIODASDENABLE, &format_params);
 		if (rc)
 			ERRMSG_EXIT(EXIT_FAILURE,
 				    "%s: (signal handler) IOCTL BIODASDENABLE "
@@ -107,13 +118,16 @@ static void program_interrupt_signal (int sig)
  * check given device name for blanks and some special characters
  * or create a devfs filename in case the devno was specified
  */
-static void get_device_name(dasdfmt_info_t *info, char *name)
+static void get_device_name(dasdfmt_info_t *info, char *name, int argc, char * argv[])
 {
-	if ((info->node_specified + info->devno_specified) > 1) 
+	if (((info->node_specified + info->devno_specified) > 1) ||
+            ((info->node_specified + info->devno_specified) > 0 &&
+              info->device_id < argc)) 
 		ERRMSG_EXIT(EXIT_MISUSE,"%s: Device can only specified once! "
 			    "(%#04x or %s)\n", prog_name, info->devno, name);
 
-	if ((info->node_specified + info->devno_specified) == 0) 
+	if ((info->node_specified + info->devno_specified) == 0 &&
+            info->device_id >= argc) 
 		ERRMSG_EXIT(EXIT_MISUSE,"%s: No device specified!\n", 
 			    prog_name);
 
@@ -124,6 +138,9 @@ static void get_device_name(dasdfmt_info_t *info, char *name)
 
 	if (info->devno_specified)
 		sprintf(info->devname, "/dev/dasd/%04x/device", info->devno);
+	else if (info->device_id < argc) {
+		strcpy(info->devname, argv[info->device_id]);
+        }
 	else {
 		if ((strchr(name, ' ') != NULL)||(strchr(name, '#') != NULL)||
 		    (strchr(name, '[') != NULL)||(strchr(name, ']') != NULL)||
@@ -135,8 +152,8 @@ static void get_device_name(dasdfmt_info_t *info, char *name)
 				    "blanks or special characters!\n", 
 				    prog_name);
 
-		strncpy(info->devname, name, LINE_LENGTH - 1);
-		info->devname[LINE_LENGTH - 1] = '\0';
+		strncpy(info->devname, name, PATH_MAX - 1);
+		info->devname[PATH_MAX - 1] = '\0';
 	}
 }
 
@@ -162,6 +179,8 @@ static void init_info(dasdfmt_info_t *info)
 	info->blksize_specified = 0;
 	info->node_specified    = 0;
 	info->devno_specified   = 0;
+	info->device_id         = 0;
+	info->keep_volser	= 0;
 }
 
 
@@ -171,12 +190,27 @@ static void init_info(dasdfmt_info_t *info)
 static void check_disk(dasdfmt_info_t *info)
 {
 	dasd_information_t dasd_info;
+	int ro, errno_save;
 
-	if (ioctl(filedes, BIODASDINFO, &dasd_info) != 0) {
+	if (ioctl(filedes, BLKROGET, &ro) != 0) {
+		errno_save = errno;
 		close(filedes);
 		ERRMSG_EXIT(EXIT_FAILURE,
-			    "%s: the ioctl call to retrieve device information"
-			    " failed (%s)\n", prog_name, strerror(errno));
+			    "%s: the ioctl call to retrieve read/write "
+			    "status information failed (%s)\n",
+			    prog_name, strerror(errno_save));
+	}
+	if (ro) {
+		ERRMSG_EXIT(EXIT_FAILURE, "Disk is read only!\n");
+	}
+
+	if (ioctl(filedes, BIODASDINFO, &dasd_info) != 0) {
+		errno_save = errno;
+		close(filedes);
+		ERRMSG_EXIT(EXIT_FAILURE,
+			    "%s: the ioctl call to retrieve device "
+			    "information failed (%s)\n",
+			    prog_name, strerror(errno_save));
 	}
 
 	if (!info->force)
@@ -200,40 +234,6 @@ static void check_disk(dasdfmt_info_t *info)
 }                                    
 
 
-/*
- * check for the API version
- */
-static void check_api_version (char *device)
-{
-        int api;
- 
-        /* get dasd driver API version */
-	if (ioctl(filedes, DASDAPIVER, &api) != 0) {
-                close(filedes);
-                ERRMSG_EXIT(EXIT_FAILURE,
-                            "%s: the ioctl call to get the API version "
-			    "returned with the following error message:\n%s\n",
-			    prog_name, strerror(errno));
-        }
- 
-	/* check API versions */
-	if (api < DASD_MIN_API_VERSION) {
-                ERRMSG_EXIT(EXIT_FAILURE,"%s: API version mismatch\n"
-			    "The current API version '%d' doesn't match "
-			    "dasd driver API version '%d'!\n", 
-			    prog_name, DASD_MIN_API_VERSION, api);          
-
-        }
-
-	if (api == 1) {
-                ERRMSG_EXIT(EXIT_FAILURE,"%s: API version mismatch\n"
-			    "The dasd driver API version '1' is not "
-			    "supported!\n", prog_name);          
-
-        }
-}                      
-
-
 /*
  * check the volume serial for special 
  * characters  and move blanks to the end
@@ -373,6 +373,44 @@ static void dasdfmt_print_info(dasdfmt_info_t *info, volume_label_t *vlabel,
 }
 
 
+/*
+ * get volser
+ */
+static int dasdfmt_get_volser(char * devname, char * volser)
+{
+	dasd_information_t  dasd_info;
+	int blksize;
+	int f;
+	volume_label_t vlabel;
+
+	if ((f = open(devname, O_RDONLY)) == -1)
+		ERRMSG_EXIT(EXIT_FAILURE,"%s: Unable to open device %s: %s\n", 
+			    prog_name, devname, strerror(errno));
+
+	if (ioctl(f, BIODASDINFO, &dasd_info) != 0)
+		ERRMSG_EXIT(EXIT_FAILURE, "%s: (label pos) IOCTL BIODASD"
+			    "INFO failed (%s).\n",prog_name, strerror(errno));
+
+	if (ioctl(f, BLKSSZGET, &blksize) != 0)
+		ERRMSG_EXIT(EXIT_FAILURE, "%s: (label pos) IOCTL BLKSSZGET "
+			    "failed (%s).\n", prog_name, strerror(errno));
+
+	if (close(f) != 0)
+		ERRMSG("%s: error during close: %s\ncontinuing...\n", 
+		       prog_name, strerror(errno));
+
+	if ((strncmp(dasd_info.type, "ECKD", 4) == 0) &&
+	    (!dasd_info.FBA_layout)) {
+		/* OS/390 and zOS compatible disk layout */
+		vtoc_read_volume_label(devname, dasd_info.label_block * blksize, &vlabel);
+		vtoc_volume_label_get_volser(&vlabel, volser);
+		return 0;
+        }
+	else {
+		return -1;
+	}
+}
+	
 /*
  * do all the labeling (volume label and initial VTOC)
  */
@@ -668,7 +706,7 @@ static void do_format_dasd(dasdfmt_info_t *info, format_data_t *p,
 	p->stop_unit  = (drive_geo.cylinders * drive_geo.heads) - 1;
 			
         if (!info->writenolabel) {
-        	if (!info->labelspec) {
+        	if (!info->labelspec && !info->keep_volser) {
 			char buf[7];
 
 	               	sprintf(buf, "0X%04x", info->devno);
@@ -730,8 +768,9 @@ int main(int argc,char *argv[])
 {
 	dasdfmt_info_t info;
 	volume_label_t vlabel;
+	char old_volser[7];
 
-	char dev_filename[LINE_LENGTH];
+	char dev_filename[PATH_MAX];
 	char str[ERR_LENGTH];
 	char buf[7];
 
@@ -765,7 +804,6 @@ int main(int argc,char *argv[])
 	{
 		rc=getopt_long(argc, argv, dasdfmt_getopt_string,
 			       dasdfmt_getopt_long_options, &index);
-        	if (rc==-1) break; // exit loop if finished
 
 		switch (rc) 
 		{
@@ -828,7 +866,7 @@ int main(int argc,char *argv[])
 			exit_usage(0);
 
 		case 'V':
-			printf("%s version %s\n", prog_name, VERSION_NO);
+			print_version();
 			exit(0);
 
 		case 'l':
@@ -861,10 +899,18 @@ int main(int argc,char *argv[])
 			break;
 		
 		case 'f' :
-			strncpy(dev_filename, optarg, LINE_LENGTH);
+			strncpy(dev_filename, optarg, PATH_MAX);
 			info.node_specified=1;
 			break;
+		case 'k' :
+			info.keep_volser=1;
+			break;
+		case -1:
+			/* End of options string - start of devices list */
+			info.device_id = optind;
+			break;
 		}
+        	if (rc==-1) break; // exit loop if finished
 	}
 
 	CHECK_SPEC_MAX_ONCE(info.blksize_specified, "blocksize");
@@ -880,16 +926,34 @@ int main(int argc,char *argv[])
 	if (info.print_hashmarks)
 		PARSE_PARAM_INTO(info.hashstep, hashstep_str,10,"hashstep");
 
-	get_device_name(&info, dev_filename);
+	get_device_name(&info, dev_filename, argc, argv);
 
         if (!info.blksize_specified)
                 format_params = ask_user_for_blksize(format_params);
 
+	if (info.keep_volser) {
+		if(info.labelspec) {
+			ERRMSG_EXIT(EXIT_MISUSE,"%s: The -k and -l options are mutually exclusive\n",
+			       prog_name);
+		}
+		if(format_params.intensity == 0x00) {
+			printf("WARNING: VOLSER cannot be kept " \
+			       "when using the ldl format!\n");
+			exit(1);
+		}
+			
+		if(dasdfmt_get_volser(info.devname, old_volser) == 0)
+			vtoc_volume_label_set_volser(&vlabel, old_volser);
+		else
+			ERRMSG_EXIT(EXIT_FAILURE,"%s: VOLSER not found on device %s\n", 
+			       prog_name, info.devname);
+			
+	}
+
 	if ((filedes = open(info.devname, O_RDWR)) == -1)
 		ERRMSG_EXIT(EXIT_FAILURE,"%s: Unable to open device %s: %s\n", 
 			    prog_name, info.devname, strerror(errno));
 
-	check_api_version(info.devname);
 	check_disk(&info);
 
 	if (check_param(str, &format_params) < 0)
