@@ -4,12 +4,6 @@
  *                  Volker Sameske <sameske@de.ibm.com>
  * Bugreports.to..: <Linux390@de.ibm.com>
  * (C) IBM Corporation, IBM Deutschland Entwicklung GmbH, 2002
- *
- * History of changes (starts March 2002)
- * 2002-03-12 initial
- * 2002-03-20 moved some definitions from dasdfmt.c
- *            removed some useless defines
- * 2002-05-10 changed API handling
  */
 
 #ifndef DASDFMT_H
@@ -30,6 +24,7 @@
 #include <mntent.h>
 #include <ctype.h>
 #include <signal.h>
+#include <limits.h>
 
 /****************************************************************************
  * SECTION: Definition needed for DASD-API (see dasd.h)                     *
@@ -93,8 +88,6 @@ typedef struct format_data_t {
 /* Enable the volume (for Linux) */
 #define BIODASDENABLE  _IO(DASD_IOCTL_LETTER,1)  
 
-/* retrieve API version number */
-#define DASDAPIVER     _IOR(DASD_IOCTL_LETTER,0,int)
 /* Get information on a dasd device (enhanced) */
 #define BIODASDINFO   _IOR(DASD_IOCTL_LETTER,1,dasd_information_t)
 
@@ -108,6 +101,8 @@ typedef struct format_data_t {
 #define BLKRRPART  _IO(0x12,95)	
 /* get block device sector size */
 #define BLKSSZGET  _IO(0x12,104)
+/* get read-only status (0 = read_write) */
+#define BLKROGET   _IO(0x12,94)
 
 /****************************************************************************
  * SECTION: Definition from hdreq.h                                         *
@@ -129,9 +124,7 @@ struct hd_geometry {
 
 #define DASD_PARTN_BITS 2
 
-#define DASD_MIN_API_VERSION 0
-
-#define VERSION_NO "1.06"
+#define VERSION_NO "1.07"
 
 #define EXIT_MISUSE 1
 #define EXIT_BUSY   2
@@ -155,7 +148,7 @@ struct hd_geometry {
 	if (*endptr) ERRMSG_EXIT(EXIT_MISUSE,"%s: " str " "    \
 	"is in invalid format\n",prog_name);}
 
-#define dasdfmt_getopt_string "b:n:l:f:d:m:hpLty?vVF"
+#define dasdfmt_getopt_string "b:n:l:f:d:m:hpLty?vVFk"
 
 static struct option dasdfmt_getopt_long_options[]=
 {
@@ -171,6 +164,7 @@ static struct option dasdfmt_getopt_long_options[]=
         { "device",      1, 0, 'f'},
         { "blocksize",   1, 0, 'b'},
         { "help",        0, 0, '?'},
+        { "keep_volser", 0, 0, 'k'},
         {0, 0, 0, 0}
 };
 
@@ -192,7 +186,7 @@ typedef struct bootstrap2 {
 
 typedef struct dasdfmt_info {
         int   devno;
-        char  devname[LINE_LENGTH];
+        char  devname[PATH_MAX];
         int   usage_count;
         int   verbosity;
         int   testmode;
@@ -206,6 +200,8 @@ typedef struct dasdfmt_info {
         int   blksize_specified;
         int   node_specified;
         int   devno_specified;
+        int   device_id;
+        int   keep_volser;
 } dasdfmt_info_t;
 
 
