@@ -369,6 +369,7 @@ void vtoc_volume_label_set_volser (volume_label_t *vlabel, char *volser)
 char * vtoc_volume_label_get_volser (volume_label_t *vlabel, char *volser)
 {
 	vtoc_ebcdic_dec(vlabel->volid, volser, VOLSER_LENGTH);
+	volser[VOLSER_LENGTH] = '\0';
 	return volser;
 }
 
@@ -408,6 +409,7 @@ void vtoc_volume_label_set_label (volume_label_t *vlabel, char *lbl)
 char * vtoc_volume_label_get_label (volume_label_t *vlabel, char *lbl)
 {
 	vtoc_ebcdic_dec(vlabel->vollbl, lbl, 4);
+	lbl[4] = '\0';
 	return lbl;
 }
 
