@@ -1772,6 +1772,23 @@ int get_time( char* buffer )
   
 } /* end get_time () */
 
+static const char contact_information[] = "linux390@de.ibm.com";
+
+static const char* usage_text[] = {
+"Usage:  osasnmpd [-h] [-v] [-l LOGFILE] [-A] [-f] [-L] [-P PIDFILE]",
+"                 [-x SOCKADDR]",
+"",
+"-h, --help              This usage message",
+"-v, --version           Version information",
+"-l, --logfile LOGFILE   Print warnings/messages to LOGFILE",
+"-A, --append            Append to the logfile rather than truncating it",
+"-L, --stderrlog         Print warnings/messages to stdout/err",
+"-f, --nofork            Do not fork() from the calling shell",
+"-P, --pidfile PIDFILE   Save the process ID of the subagent in PIDFILE",
+"-x, --sockaddr SOCKADDR Bind AgentX port to this address",
+""
+};
+
 /**********************************************************************
  * usage():
  *  prints help message with descriptions of available parameters    
@@ -1780,21 +1797,10 @@ int get_time( char* buffer )
  *********************************************************************/
 void usage()
 {
-	
-  printf( "\nUsage:  osasnmpd [-h] [-v] [-l LOGFILE] [-A] [-f] [-L] [-P PIDFILE]" );
-  printf( "\n\t\t [-x SOCKADDR]" );
-  printf( "\n\n\tVersion:     %s", RELEASE_STRING );
-  printf( "\n\tDescription: IBM OSA-E SNMP subagent for ucd-snmp 4.2.x series\n"); 
-  printf( "\n-h\t\tThis usage message" );
-  printf( "\n-v\t\tVersion information" );
-  printf( "\n-l LOGFILE\tPrint warnings/messages to LOGFILE" );
-  printf( "\n\t\t(By default LOGFILE=%s)", OSAE_LOGFILE );
-  printf( "\n-A\t\tAppend to the logfile rather than truncating it" );
-  printf( "\n-L\t\tPrint warnings/messages to stdout/err" );
-  printf( "\n-f\t\tDo not fork() from the calling shell" );
-  printf( "\n-P PIDFILE\tSave the process ID of the subagent in PIDFILE"); 
-  printf( "\n-x SOCKADDR\tBind AgentX port to this address");
-  printf( "\n\n" );
-  exit(0);
-  
+	unsigned int i;
+
+	for (i=0; i < sizeof(usage_text) / sizeof(usage_text[0]); i++)
+		printf("%s\n", usage_text[i]);
+	printf("Please report bugs to: %s\n", contact_information);
+	exit(0);
 } /* end usage() */
