@@ -5,7 +5,7 @@
  *  to a dump device. The written dump can be analyzed afterwards
  *  with the dump analysis tool lcrash.
  *  
- *    Copyright (C) 2003 IBM Deutschland Entwicklung GmbH, IBM Corporation
+ *    Copyright (C) 2003-2005 IBM Deutschland Entwicklung GmbH, IBM Corporation
  *    Author(s): Michael Holzheu <holzheu@de.ibm.com>
  */
 
@@ -14,7 +14,7 @@
 #include <linux/kernel.h>
 #include <linux/version.h>
 #include <linux/init.h>
-#include <linux/locks.h>
+#include <linux/spinlock.h>
 #include <linux/slab.h>
 #include <linux/module.h>
 #include <linux/blkdev.h>
@@ -90,7 +90,6 @@ struct {
 #define MIN(a,b) ( (a)<(b) ? (a):(b) )
 #define MAX(a,b) ( (a)>(b) ? (a):(b) )
 
-#define STCK(x) asm volatile ("STCK 0(%0)" : : "a" (&(x)) : "memory", "cc")
 #define STIDP(x) asm volatile ("STIDP 0(%0)" : : "a" (&(x)) : "memory" ,"cc")
 
 #define DUMP_PRINT_TRACE(x...) if(dump.debug_level > 3) {printk( KERN_ALERT "K_TRACE: " x );}
@@ -136,15 +135,15 @@ typedef union {
 		__u32 acc_regs[16];
 		__u64 ctrl_regs[16];
 	}  __attribute__((packed)) esame;
-} dump_cpu_info_t __attribute__((packed));
+} dump_cpu_info_t;
 
 /* dump arch info */
 
 typedef struct _dump_arch {
-	dump_arch_id_t arch;	    /* arch id */
-	__u64	    save_area_base; /* save area base relative to prefix page */
-	__u32	    save_area_size; /* size fo save area */
-	dump_cpu_info_t* cpu_info;  /* cpu-info array with registers */
+	dump_arch_id_t   arch;	         /* arch id */
+	unsigned long    save_area_base; /* save area base rel. to pref. page */
+	__u32	         save_area_size; /* size fo save area */
+	dump_cpu_info_t* cpu_info;       /* cpu-info array with registers */
 } dump_arch_t;
 
 /*******************************************************************
@@ -152,18 +151,18 @@ typedef struct _dump_arch {
  *******************************************************************/
  
 static void dump_cpu_stop_and_store_status(int cpu);
-static void dump_cpu_start(int cpu);
 int dump_init(void);
-static int dump_cleanup(void);
-
+void dump_exit(void);
 static int zcore_init(void);
-static int zcore_cleanup(void);
+static void zcore_exit(void);
 static int zcore_open(struct inode * inode, struct file * filp);
 static loff_t zcore_lseek(struct file * file, loff_t offset, int orig);
-static ssize_t zcore_read(struct file * file, char * buf, size_t count, loff_t *ppos);
-static int zcore_ioctl(struct inode *inode, struct file *filp, unsigned int cmd, unsigned long arg) ;
+static ssize_t zcore_read(struct file * file, char * buf, size_t count,
+		loff_t *ppos);
+static int zcore_ioctl(struct inode *inode, struct file *filp, unsigned int cmd,
+		unsigned long arg) ;
 
-static void sdias_init(void);
+static int sdias_init(void);
 static int sdias_get_nr_blocks(void);
 static int sdias_copy_blocks(void* target,int start_blk, int nr_blks);
 static int sdias_memcpy(char* target, char* src, int count);
@@ -174,7 +173,7 @@ static int sdias_memcpy(char* target, char* src, int count);
 
 extern unsigned long memory_size;   /* Mem size */
 extern char          *ldipl_xml;    /* xml parameter passed by ldipl */
-extern __u16         boot_cpu_addr; /* physical number of boot cpu */
+extern volatile int __cpu_logical_map[];
 
 /*******************************************************************
  * globals
@@ -238,7 +237,8 @@ memcpy_real(void *dest, const void *src, size_t count)
 	return ret;
 }
 
-#define memcpy_absolute memcpy_real /* This is correct since we set the prefix page to 0 */
+#define memcpy_absolute memcpy_real /* This is correct since we set the */
+                                    /* prefix page to 0 */
 
 /* 
  * Send a signal to a logical cpu number 
@@ -333,6 +333,9 @@ dump_smp_count_cpus(void)
 	int curr_cpu;
 	int num_cpus = 1;
 	int rc;
+        __u16 boot_cpu_addr = __cpu_logical_map[0]; /* physical number of */
+                                                    /* cpu */
+
 
 	dump.cpu_logical_map[0] = boot_cpu_addr;
 	for (curr_cpu = 0; curr_cpu <= 65535; curr_cpu++) {
@@ -359,25 +362,41 @@ static void
 dump_print_cpu_info_esa(dump_cpu_info_t* cpu_info)
 {
 	int i;
-	DUMP_PRINT_TRACE("psw   : %08x %08x\n",*((int*)cpu_info->esa.psw),*((int*)&(cpu_info->esa.psw[4])));
+	DUMP_PRINT_TRACE("psw   : %08x %08x\n",*((int*)cpu_info->esa.psw),
+		*((int*)&(cpu_info->esa.psw[4])));
 	DUMP_PRINT_TRACE("prefix: %08x\n",cpu_info->esa.pref_reg);
-	DUMP_PRINT_TRACE("clk   : %016lx\n",cpu_info->esa.timer);
-	DUMP_PRINT_TRACE("clkcmp: %016lx\n",cpu_info->esa.clock_comparator);
+	DUMP_PRINT_TRACE("clk   : %016Lx\n",(long long)cpu_info->esa.timer);
+	DUMP_PRINT_TRACE("clkcmp: %016Lx\n",
+		(long long)cpu_info->esa.clock_comparator);
 	DUMP_PRINT_TRACE("gpregs:\n");
 	for(i = 0; i < 16; i+=4){
-		DUMP_PRINT_TRACE("%08x %08x %08x %08x\n",cpu_info->esa.gp_regs[i], cpu_info->esa.gp_regs[i+1],cpu_info->esa.gp_regs[i+2],cpu_info->esa.gp_regs[i+3]);
+		DUMP_PRINT_TRACE("%08x %08x %08x %08x\n",
+			cpu_info->esa.gp_regs[i],
+			cpu_info->esa.gp_regs[i+1],
+			cpu_info->esa.gp_regs[i+2],
+			cpu_info->esa.gp_regs[i+3]);
 	}
 	DUMP_PRINT_TRACE("accregs:\n");
 	for(i = 0; i < 16; i+=4){
-		DUMP_PRINT_TRACE("%08x %08x %08x %08x\n",cpu_info->esa.acc_regs[i], cpu_info->esa.acc_regs[i+1],cpu_info->esa.acc_regs[i+2],cpu_info->esa.acc_regs[i+3]);
+		DUMP_PRINT_TRACE("%08x %08x %08x %08x\n",
+			cpu_info->esa.acc_regs[i],
+			cpu_info->esa.acc_regs[i+1],
+			cpu_info->esa.acc_regs[i+2],
+			cpu_info->esa.acc_regs[i+3]);
 	}
 	DUMP_PRINT_TRACE("ctrl_regs:\n");
 	for(i = 0; i < 16; i+=4){
-		DUMP_PRINT_TRACE("%08x %08x %08x %08x\n",cpu_info->esa.ctrl_regs[i], cpu_info->esa.ctrl_regs[i+1],cpu_info->esa.ctrl_regs[i+2],cpu_info->esa.ctrl_regs[i+3]);
+		DUMP_PRINT_TRACE("%08x %08x %08x %08x\n",
+			cpu_info->esa.ctrl_regs[i],
+			cpu_info->esa.ctrl_regs[i+1],
+			cpu_info->esa.ctrl_regs[i+2],
+			cpu_info->esa.ctrl_regs[i+3]);
 	}
 	DUMP_PRINT_TRACE("fp_regs:\n");
 	for(i = 0; i < 4; i+=2){
-		DUMP_PRINT_TRACE("%016lx %016lx\n",cpu_info->esa.fp_regs[i], cpu_info->esa.fp_regs[i+1]);
+		DUMP_PRINT_TRACE("%016Lx %016Lx\n",
+			(long long)cpu_info->esa.fp_regs[i],
+			(long long)cpu_info->esa.fp_regs[i+1]);
 	}
 }
 
@@ -389,27 +408,40 @@ static void
 dump_print_cpu_info_esame(dump_cpu_info_t* cpu_info)
 {
 	int i;
-	DUMP_PRINT_TRACE("psw: %016lx %016lx\n",*((unsigned long*)cpu_info->esame.psw),*((unsigned long*)&(cpu_info->esame.psw[8])));
+	DUMP_PRINT_TRACE("psw: %016lx %016lx\n",
+		*((unsigned long*)cpu_info->esame.psw),
+		*((unsigned long*)&(cpu_info->esame.psw[8])));
 	DUMP_PRINT_TRACE("prefix: %08x\n",cpu_info->esame.pref_reg);
-	DUMP_PRINT_TRACE("clk   : %016lx\n",cpu_info->esame.timer);
-	DUMP_PRINT_TRACE("clkcmp: %016lx\n",cpu_info->esame.clock_comparator);
+	DUMP_PRINT_TRACE("clk   : %016Lx\n",(long long)cpu_info->esame.timer);
+	DUMP_PRINT_TRACE("clkcmp: %016Lx\n",
+		(long long)cpu_info->esame.clock_comparator);
 	DUMP_PRINT_TRACE("fpctrl: %04x\n",cpu_info->esame.fp_ctrl_reg);
 	DUMP_PRINT_TRACE("todreg: %04x\n",cpu_info->esame.tod_reg);
 	DUMP_PRINT_TRACE("gpregs:\n");
 	for(i = 0; i < 16; i+=2){
-		DUMP_PRINT_TRACE("%016lx %016lx\n",cpu_info->esame.gp_regs[i], cpu_info->esame.gp_regs[i+1]);
+		DUMP_PRINT_TRACE("%016Lx %016Lx\n",
+			(long long)cpu_info->esame.gp_regs[i],
+			(long long)cpu_info->esame.gp_regs[i+1]);
 	}
 	DUMP_PRINT_TRACE("accregs:\n");
 	for(i = 0; i < 16; i+=4){
-		DUMP_PRINT_TRACE("%08x %08x %08x %08x\n",cpu_info->esame.acc_regs[i], cpu_info->esame.acc_regs[i+1],cpu_info->esame.acc_regs[i+2],cpu_info->esame.acc_regs[i+3]);
+		DUMP_PRINT_TRACE("%08x %08x %08x %08x\n",
+			cpu_info->esame.acc_regs[i],
+			cpu_info->esame.acc_regs[i+1],
+			cpu_info->esame.acc_regs[i+2],
+			cpu_info->esame.acc_regs[i+3]);
 	}
 	DUMP_PRINT_TRACE("ctrl_regs:\n");
 	for(i = 0; i < 16; i+=2){
-		DUMP_PRINT_TRACE("%016lx %016lx\n",cpu_info->esame.ctrl_regs[i], cpu_info->esame.ctrl_regs[i+1]);
+		DUMP_PRINT_TRACE("%016Lx %016Lx\n",
+			(long long)cpu_info->esame.ctrl_regs[i],
+			(long long)cpu_info->esame.ctrl_regs[i+1]);
 	}
 	DUMP_PRINT_TRACE("fp_regs:\n");
 	for(i = 0; i < 16; i+=2){
-		DUMP_PRINT_TRACE("%016lx %016lx\n",cpu_info->esame.fp_regs[i], cpu_info->esame.fp_regs[i+1]);
+		DUMP_PRINT_TRACE("%016Lx %016Lx\n",
+			(long long)cpu_info->esame.fp_regs[i],
+			(long long)cpu_info->esame.fp_regs[i+1]);
 	}
 }
 
@@ -419,10 +451,11 @@ dump_print_cpu_info_esame(dump_cpu_info_t* cpu_info)
 static void 
 dump_print_glob_info(void)
 {
-	DUMP_PRINT_TRACE("Architecture		   : %s\n",((dump.arch_info.arch == esa) ? "esa" : "esame"));
-	DUMP_PRINT_TRACE("Nummber of cpus		   : %i\n",dump.nr_cpus);
-	DUMP_PRINT_TRACE("Memory size		   : %li\n",dump.mem_size);
-	DUMP_PRINT_TRACE("HSA Memory size            : %li\n",dump.sa_mem_size);
+	DUMP_PRINT_TRACE("Architecture    : %s\n",
+		((dump.arch_info.arch == esa) ? "esa" : "esame"));
+	DUMP_PRINT_TRACE("Nummber of cpus : %i\n", dump.nr_cpus);
+	DUMP_PRINT_TRACE("Memory size     : %li\n",dump.mem_size);
+	DUMP_PRINT_TRACE("HSA Memory size : %li\n",dump.sa_mem_size);
 }
 
 /*
@@ -435,10 +468,11 @@ dump_copy_esame_to_esa(dump_cpu_info_t* esa_info, dump_cpu_info_t* esame_info,in
 
 	DUMP_PRINT_TRACE("dump_copy_esame_to_esa:");
 	for(i = 0; i < 16; i++){
-		esa_info->esa.gp_regs[i]   = esame_info->esame.gp_regs[i] & 0x00000000ffffffff;
+		esa_info->esa.gp_regs[i]   = esame_info->esame.gp_regs[i] & 
+						0x00000000ffffffff;
 		esa_info->esa.acc_regs[i]  = esame_info->esame.acc_regs[i];
-		esa_info->esa.ctrl_regs[i] = esame_info->esame.ctrl_regs[i] & 0x00000000ffffffff;
-		// DUMP_PRINT_TRACE("gp(%i) = %lx / %x\n",i,esame_info->esame.gp_regs[i],esa_info->esa.gp_regs[i]);
+		esa_info->esa.ctrl_regs[i] = esame_info->esame.ctrl_regs[i] & 
+						0x00000000ffffffff;
 	}
 	/* locore for 31 bit has only space for fpregs 0,2,4,6 */
 	esa_info->esa.fp_regs[0] = esame_info->esame.fp_regs[0];
@@ -449,7 +483,6 @@ dump_copy_esame_to_esa(dump_cpu_info_t* esa_info, dump_cpu_info_t* esame_info,in
 	esa_info->esa.psw[1] |= 0x8; /* set bit 12 */
 	memcpy(&(esa_info->esa.psw[4]),&(esame_info->esame.psw[12]),4);
 	esa_info->esa.psw[4] |= 0x80; /* set (31bit) addressing bit */
-	// esa_info->esa.pref_reg	= esame_info->esame.pref_reg;
 	esa_info->esa.pref_reg  = dump_prefix_array[cpu-1];
 	DUMP_PRINT_TRACE("CPU %i - prefix = %x\n",cpu,esa_info->esa.pref_reg);
 	esa_info->esa.timer = esame_info->esame.timer;
@@ -467,7 +500,8 @@ dump_get_cpu_info(int nr_cpus)
 
 	info  = kmalloc(nr_cpus * sizeof(dump_cpu_info_t), GFP_KERNEL);
 	if(!info) {
-		DUMP_PRINT_ERR("kmalloc failed: %s: %i\n",__FUNCTION__,__LINE__);
+		DUMP_PRINT_ERR("kmalloc failed: %s: %i\n",__FUNCTION__,
+				__LINE__);
 		goto error;
 	}
 
@@ -475,19 +509,24 @@ dump_get_cpu_info(int nr_cpus)
 
 	switch(dump.arch_info.arch){
 		case esa:
-			if(sdias_memcpy((char*)&(info[dump_cpu].esa),(char*)dump.arch_info.save_area_base,sizeof(info[0].esa)) < 0){
+			if(sdias_memcpy((char*)&(info[dump_cpu].esa),
+				(char*)dump.arch_info.save_area_base,
+				sizeof(info[0].esa)) < 0){
 				DUMP_PRINT_ERR("could not copy from HSA\n");
 				goto error;
 			}
 			break;
 		case esame:
-			if(sdias_memcpy((char*)&(info[dump_cpu].esame),(void*)dump.arch_info.save_area_base,sizeof(info[0].esame)) < 0){
+			if(sdias_memcpy((char*)&(info[dump_cpu].esame),
+				(void*)dump.arch_info.save_area_base,
+				sizeof(info[0].esame)) < 0){
 				DUMP_PRINT_ERR("could not copy from HSA\n");
 				goto error;
 			}
 			break;
 		default:
-			DUMP_PRINT_ERR("dump: unknown arch %x\n",dump.arch_info.arch);
+			DUMP_PRINT_ERR("dump: unknown arch %x\n",
+				dump.arch_info.arch);
 			goto error;
 	}
 
@@ -505,20 +544,29 @@ dump_get_cpu_info(int nr_cpus)
 			
 		dump_cpu_stop_and_store_status(i);
 		switch(dump.arch_info.arch){
+#ifdef __s390x__ 
 			dump_cpu_info_t tmp_info;
+#endif
 			case esa:
-#ifdef __s390x__ /* 64 bit */
-				memcpy_absolute(&(tmp_info.esame),(char*)DUMP_SAVE_AREA_BASE_ESAME ,sizeof(tmp_info.esame));
+#ifdef __s390x__
+				memcpy_absolute(&(tmp_info.esame),
+					(char*)DUMP_SAVE_AREA_BASE_ESAME ,
+					sizeof(tmp_info.esame));
 				dump_copy_esame_to_esa(&info[i],&tmp_info,i);
-#else /* 32 bit */
-				memcpy_absolute(&(info[i].esa),(char*)dump.arch_info.save_area_base,sizeof(info[0].esa));
-#endif /* __s390x__ */
+#else
+				memcpy_absolute(&(info[i].esa),
+					(char*)dump.arch_info.save_area_base,
+					sizeof(info[0].esa));
+#endif
 				break;
 			case esame:
-				memcpy_absolute(&(info[i].esame),(char*)dump.arch_info.save_area_base,sizeof(info[0].esame));
+				memcpy_absolute(&(info[i].esame),
+					(char*)dump.arch_info.save_area_base,
+					sizeof(info[0].esame));
 				break;
 			default:
-				DUMP_PRINT_ERR("dump: unknown arch %x\n",dump.arch_info.arch);
+				DUMP_PRINT_ERR("dump: unknown arch %x\n",
+					dump.arch_info.arch);
 				goto error;
 		}
 		DUMP_PRINT_TRACE("cpu: %i\n",i);
@@ -536,17 +584,7 @@ error:
 	return NULL;
 }
 
-/*
- * Free CPU info struct
- */
-
-static void 
-dump_free_cpu_info(dump_cpu_info_t* info)
-{
-	kfree(info);
-}
-
-void setup_dump(char* cmdline)
+void setup_dump(char* cmdline, unsigned short *init_devno)
 {
 	char str[100] = {0};
 #ifdef __s390x__
@@ -554,7 +592,8 @@ void setup_dump(char* cmdline)
 #else
 	printk("System Dumper s390 starting...\n");
 #endif
-	sprintf(str," root=/dev/ram0 init=/linuxrc rw mem=%liM maxcpus=1",DUMP_MEM_SIZE_MB);
+	sprintf(str," root=/dev/ram0 rw mem=%liM maxcpus=1 cio_ignore=all,!0.0.%04x",
+                DUMP_MEM_SIZE_MB, *init_devno);
 	strcat(cmdline,str);
 	console_loglevel = DEBUG_LEVEL_DEFAULT;
 	dump.debug_level = DEBUG_LEVEL_DEFAULT;
@@ -564,18 +603,12 @@ void setup_dump(char* cmdline)
  * Initialize dump globals for a given architecture
  */
 
-int 
+int __init
 dump_init(void)
 {
 	unsigned char arch_id = 0x88;
-	// memset(&dump,0,sizeof(dump));
 	long hsa_nr_of_blocks;
 	int rc = 0;
-	int i = 0;
-	while((dump_prefix_array[i] != 0) && (i < DUMP_MAX_CPUS) 
-              && (dump_prefix_array[i] != 0xffffffff)){
-		i++;
-	}
 
         dump.dbf = debug_register("dump", 1, 1, 8);
 	if(dump.dbf)
@@ -587,7 +620,10 @@ dump_init(void)
 	DUMP_PRINT_TRACE("DUMP init\n");
 	DUMP_PRINT_TRACE("xml: %s\n",ldipl_xml);
 
-	sdias_init();
+	if(sdias_init()){
+		DUMP_PRINT_ERR("could initialize sclp\n");
+		rc = -1; goto out;
+	}
 	hsa_nr_of_blocks = sdias_get_nr_blocks();
 	if(hsa_nr_of_blocks == -1){
 		DUMP_PRINT_ERR("could not determine HSA size\n");
@@ -595,7 +631,7 @@ dump_init(void)
 		dump.error_code = ZCORE_ERR_HSA_FEATURE;
 		rc = -1; goto out;
 	}
-	dump.sa_mem_size = (hsa_nr_of_blocks - 1) *  4096; // XXX -1 ???
+	dump.sa_mem_size = (hsa_nr_of_blocks - 1) *  4096;
 
 	if(dump.sa_mem_size < (DUMP_MEM_SIZE_MB * 1024 * 1024)){
 		DUMP_PRINT_ERR("hsa size too small! Should be at least %li - is %li\n",(DUMP_MEM_SIZE_MB * 1024UL * 1024UL), dump.sa_mem_size);
@@ -608,7 +644,8 @@ dump_init(void)
 
 	/* check arch of operating system */
 
-	DUMP_PRINT_TRACE("prefix of dump cpu: %lx\n",(unsigned long)lowcore_ptr[0]);
+	DUMP_PRINT_TRACE("prefix of dump cpu: %lx\n",
+				(unsigned long)lowcore_ptr[0]);
 #ifdef __s390x__ /* 64 bit */
 
 	DUMP_PRINT_TRACE("64 bit System dumper\n");
@@ -631,20 +668,23 @@ dump_init(void)
 	memcpy(lowcore_ptr[0],0,LOWCORE_SIZE);
 	set_prefix(0);
 	lowcore_ptr[0] = 0;
-	DUMP_PRINT_TRACE("prefix of dump cpu: %lx\n",(unsigned long)lowcore_ptr[0]);
+	DUMP_PRINT_TRACE("prefix of dump cpu: %lx\n",
+		(unsigned long)lowcore_ptr[0]);
 
 	switch(arch_id){
 	case DUMP_LC_ARCH_MODE_ESAME:
 		DUMP_PRINT("DETECTED 'ESAME (64 bit) OS'...\n");
 		dump.arch_info.arch = esame;
 		dump.arch_info.save_area_base = DUMP_SAVE_AREA_BASE_ESAME;
-		dump.arch_info.save_area_size = sizeof(dump.arch_info.cpu_info[0].esame);
+		dump.arch_info.save_area_size = 
+			sizeof(dump.arch_info.cpu_info[0].esame);
 		break;
 	case DUMP_LC_ARCH_MODE_ESA:
 		DUMP_PRINT("DETECTED 'ESA (32 bit) OS'...\n");
 		dump.arch_info.arch = esa;
 		dump.arch_info.save_area_base = DUMP_SAVE_AREA_BASE_ESA;
-		dump.arch_info.save_area_size = sizeof(dump.arch_info.cpu_info[0].esa);
+		dump.arch_info.save_area_size =
+			sizeof(dump.arch_info.cpu_info[0].esa);
 		break;
 	default:
 		DUMP_PRINT_WARN("unknown architecture 0x%x.\n",arch_id);
@@ -652,7 +692,8 @@ dump_init(void)
 		DUMP_PRINT_WARN("Assuming ESAME architecture now...\n");
 		dump.arch_info.arch = esame;
 		dump.arch_info.save_area_base = DUMP_SAVE_AREA_BASE_ESAME;
-		dump.arch_info.save_area_size = sizeof(dump.arch_info.cpu_info[0].esame);
+		dump.arch_info.save_area_size =
+			sizeof(dump.arch_info.cpu_info[0].esame);
 		break;
 	}
 
@@ -676,15 +717,14 @@ out:
 }
 
 /*
- * dump_cleanup()
+ * cleanup everything
  */
 
-static int 
-dump_cleanup(void)
+void __exit
+dump_exit(void)
 {
-	// dump_free_cpu_info();
-	zcore_cleanup();
-	return 0;
+	zcore_exit();
+	kfree(dump.arch_info.cpu_info);
 }
 
 /*
@@ -701,20 +741,6 @@ dump_cpu_stop_and_store_status(int cpu)
 	} while(ccode == sigp_busy);
 }
 
-/*
- * Restart a cpu
- */
-
-static void 
-dump_cpu_start(int cpu)
-{
-	int ccode = 0;
-
-	do {
-		ccode = dump_sigp(cpu, sigp_start);
-	} while(ccode == sigp_busy);
-}
-
 /****************************************************************************/
 /* zcore code:								    */
 /****************************************************************************/
@@ -784,15 +810,12 @@ zcore_init(void)
 	zcore_dump_header.num_pages	 = dump.mem_size / 4096;
 	STCK(zcore_dump_header.tod);
 	STIDP(zcore_dump_header.cpu_id);
-	if (devfs_register_chrdev(ZCORE_MAJOR,"zcore",&zcore_fops)){
-		DUMP_PRINT_ERR("unable to get major %d for memory devs\n", MEM_MAJOR);
+	if (register_chrdev(ZCORE_MAJOR,"zcore",&zcore_fops)){
+		DUMP_PRINT_ERR("unable to get major %d for memory devs\n",
+			MEM_MAJOR);
 		rc = -1; goto out;
 
 	}
-	devfs_register (NULL, "zcore", DEVFS_FL_NONE,
-			ZCORE_MAJOR, 0,
-			S_IRUGO | S_IFCHR,
-			&zcore_fops, NULL);
 out:
 	return rc;
 }
@@ -800,12 +823,12 @@ out:
 /*
  *  Unregister zcore device
  */
-static int 
-zcore_cleanup(void)
+static void 
+zcore_exit(void)
 {
-	devfs_unregister_chrdev (ZCORE_MAJOR, "zcore");
-	return 0;
-} 
+	unregister_chrdev (ZCORE_MAJOR, "zcore");
+}
+
 
 /*
  * Copy lowocre info to memory, if necessary
@@ -835,8 +858,9 @@ zcore_add_lowcores_to_output(char* buf, size_t count, unsigned long mem_start)
 			continue;
 		new_start= MAX(mem_start,save_start);
 		new_end  = MIN(mem_end,save_end);
-		// DUMP_PRINT_ERR("copy lowcore: cpu=%i (%lx %lx) (%lx %lx) (%lx %lx)\n",i, mem_start, mem_end, save_start, save_end, new_start, new_end); 
-		if(copy_to_user((buf + (new_start - mem_start)),((char*)&(dump.arch_info.cpu_info[i])) + (new_start - save_start),(new_end-new_start)))
+		if(copy_to_user((buf + (new_start - mem_start)),
+			((char*)&(dump.arch_info.cpu_info[i])) +
+			(new_start - save_start),(new_end-new_start)))
 			return -EFAULT;
 	}
 	return 0;
@@ -862,9 +886,9 @@ zcore_read(struct file * file, char * buf, size_t count, loff_t *ppos)
 
 	if(start_pos < KL_DUMP_HEADER_SZ_S390){
 		/* Copy dump header */
-		// DUMP_PRINT_TRACE("zcore_read: (header) : %li\n",start_pos);
+		DUMP_PRINT_TRACE("zcore_read: (header) : %li\n",start_pos);
 		end = MIN(count, KL_DUMP_HEADER_SZ_S390-start_pos);
-		if(copy_to_user(buf,&zcore_dump_header,end))
+		if(copy_to_user(buf,&zcore_dump_header + start_pos, end))
 			return -EFAULT;
 		rel_pos+=end;
 	}
@@ -872,12 +896,15 @@ zcore_read(struct file * file, char * buf, size_t count, loff_t *ppos)
 		goto out;
 
 	if (MEM_LOC(start_pos+rel_pos) < dump.sa_mem_size){
-		// DUMP_PRINT_TRACE("zcore_read: (sdias) : %li\n",(start_pos + rel_pos));
-		end = rel_pos + MIN((count-rel_pos), (dump.sa_mem_size - MEM_LOC(start_pos + rel_pos)));
+		DUMP_PRINT_TRACE("zcore_read: (sdias) : %li\n",(start_pos +
+								rel_pos));
+		end = rel_pos + MIN((count-rel_pos), (dump.sa_mem_size -
+						MEM_LOC(start_pos + rel_pos)));
 		for(;rel_pos < end;){
 			int size = MIN(sizeof(dump.buffer), (end-rel_pos));
 			int rc;
-			rc = sdias_memcpy(dump.buffer, (void*)MEM_LOC(start_pos + rel_pos) , size);
+			rc = sdias_memcpy(dump.buffer,
+				(void*)MEM_LOC(start_pos + rel_pos) , size);
 			if(rc<0){
 				DUMP_PRINT_ERR("sdias_memcpy failed\n");
 				return -EIO;
@@ -893,10 +920,14 @@ zcore_read(struct file * file, char * buf, size_t count, loff_t *ppos)
 
 	/* Copy from real mem */
 	if (MEM_LOC(start_pos+rel_pos) < dump.mem_size){
-		end = rel_pos + MIN((count-rel_pos), (dump.mem_size - MEM_LOC(start_pos + rel_pos)));
+		DUMP_PRINT_TRACE("zcore_read: (real mem) : %li\n",
+			(start_pos + rel_pos));
+		end = rel_pos + MIN((count-rel_pos), (dump.mem_size -
+			MEM_LOC(start_pos + rel_pos)));
 		for(;rel_pos < end;){
 			int size = MIN(sizeof(dump.buffer), (end-rel_pos));
-			memcpy_absolute(dump.buffer, (void*)MEM_LOC(start_pos + rel_pos) , size);
+			memcpy_absolute(dump.buffer, (void*)MEM_LOC(start_pos +
+							rel_pos) , size);
 			if (copy_to_user(buf + rel_pos, dump.buffer, size))
 				return -EFAULT;
 			rel_pos += size;
@@ -904,7 +935,8 @@ zcore_read(struct file * file, char * buf, size_t count, loff_t *ppos)
 	}
 out:
 	if(*ppos > 4096) {
-		if( zcore_add_lowcores_to_output(buf, rel_pos,MEM_LOC(*ppos)) == -EFAULT){
+		if( zcore_add_lowcores_to_output(buf, rel_pos,MEM_LOC(*ppos))
+			== -EFAULT){
 			return -EFAULT;
 		}
 	}
@@ -1002,8 +1034,10 @@ zcore_ioctl(struct inode *inode, struct file *filp,
 			break;
 		}
 		case ZCORE_IOCTL_GET_ERROR:
-			if(copy_to_user ((char *) arg, &dump.error_code, sizeof(dump.error_code))){
-				DUMP_PRINT_ERR("ZCORE_IOCTL_GET_ERROR: failed\n");
+			if(copy_to_user ((char *) arg, &dump.error_code,
+				sizeof(dump.error_code))){
+				DUMP_PRINT_ERR("ZCORE_IOCTL_GET_ERROR: "
+						"failed\n");
 				rc = -EFAULT;
 			}
 			break;
@@ -1027,13 +1061,14 @@ zcore_ioctl(struct inode *inode, struct file *filp,
 #include <linux/version.h>
 #include <asm/semaphore.h>
 #include <asm/ebcdic.h>
-#include "hwc_rw.h"
-#include "hwc.h"
+#include "sclp.h"
+#include "sclp_rw.h"
 
 #define SDIAS_RETRIES	    300
 #define SDIAS_SLEEP_TICKS   50
 
 #define SDIAS_OP_CODE 0x1c
+#define SDIAS_MASK    0x00000010
 #define EQ_STORE_DATA 0x0
 #define EQ_SIZE       0x1
 #define DI_FCP_DUMP   0x0
@@ -1043,8 +1078,13 @@ zcore_ioctl(struct inode *inode, struct file *filp,
 #define EVSTATE_NO_DATA      0x3
 #define EVSTATE_PART_STORED  0x10
 
-typedef struct _sdias_evbuf {
-	_EBUF_HEADER
+
+struct sclp_register sclp_sdias_register = {
+	.send_mask = SDIAS_MASK,
+};
+
+struct sdias_evbuf {
+	struct  evbuf_header header;
 	u8	event_qualifier;
 	u8	data_identifier;
 	u64	reserved2;
@@ -1061,12 +1101,12 @@ typedef struct _sdias_evbuf {
 	u32	lbn;
 	u16	reserved7;
 	u16	dbs;
-} __attribute__((packed)) sdias_evbuf_t;
+} __attribute__((packed));
 
-typedef struct _sdias_sccb_t {
-	_HWCB_HEADER
-	sdias_evbuf_t sdias_evbuf;
-} __attribute__((packed)) sdias_sccb_t;
+struct sdias_sccb {
+	struct sccb_header  header;
+	struct sdias_evbuf  sdias_evbuf;
+} __attribute__((packed));
 
 /* Globals */
  
@@ -1082,9 +1122,12 @@ static wait_queue_head_t sdias_wq;
  */
  
 static void 
-sdias_callback(hwc_request_t* req)
+sdias_callback(struct sclp_req* request, void* data)
 {
+	struct sdias_sccb* sccb;
+	sccb = (struct sdias_sccb*) request->sccb;
 	debug_text_event(dump.dbf,1,"callb1");
+
 	sclp_req_done = 1;
 	wake_up(&sdias_wq); /* Inform caller, that request is complete */
 	debug_text_event(dump.dbf,1,"callb2");
@@ -1096,20 +1139,20 @@ sdias_callback(hwc_request_t* req)
  */
  
 static int 
-dump_sclp_send(hwc_request_t *req)
+dump_sclp_send(struct sclp_req *req)
 {
 	int retries;
 	int rc;
-
 	debug_text_event(dump.dbf,1,"sclp1");
 	for (retries = SDIAS_RETRIES; retries; retries--) {
 		sclp_req_done = 0;
-		rc = hwc_send(req);
+		rc = sclp_add_request(req);
 		debug_text_event(dump.dbf,1,"sclp2");
 		if (rc) {
 			/* not initiated, wait some time and retry */
 			set_current_state(TASK_INTERRUPTIBLE);
-			DUMP_PRINT_TRACE("SCLP call failed (rc = %i). waiting..\n",rc);
+			DUMP_PRINT_TRACE("SCLP call failed (rc = %i). "
+					"waiting..\n",rc);
 			debug_text_event(dump.dbf,1,"FAIL");
 			schedule_timeout(SDIAS_SLEEP_TICKS);
 			debug_text_event(dump.dbf,1,"WAIT");
@@ -1118,6 +1161,10 @@ dump_sclp_send(hwc_request_t *req)
 			/* initiated, wait for completion of service call */
 			wait_event(sdias_wq,(sclp_req_done == 1));
 			debug_text_event(dump.dbf,1,"sclp4");
+			if (req->status == SCLP_REQ_FAILED){
+				DUMP_PRINT_TRACE("SCLP call failed\n");
+				continue;	
+			} 
 			goto out;
 		} 
 	}
@@ -1129,12 +1176,16 @@ out:
  * The init routine: Call before useing hsa_sdias_copy_blocks()
  */
 
-static void 
+static int
 sdias_init(void)
 {
+	
+	if(sclp_register(&sclp_sdias_register) != 0)
+		return -1;
 	debug_text_event(dump.dbf,1,"sdinit1");
 	init_waitqueue_head(&sdias_wq);
 	debug_text_event(dump.dbf,1,"sdinit2");
+	return 0;
 }
 
 /*
@@ -1142,27 +1193,28 @@ sdias_init(void)
  * - not reentrant! -
  */
 
-static sdias_sccb_t sccb __attribute__((__aligned__(4096)));
+static struct sdias_sccb sccb __attribute__((__aligned__(4096)));
 static int 
 sdias_get_nr_blocks(void)
 {
-	hwc_request_t sdias_request = {};
+	struct sclp_req sdias_request = {};
 	int rc;
 
 	debug_text_event(dump.dbf,1,"getblk1");
 	memset(&sccb,0,sizeof(sccb));
  
-	sccb.length = sizeof(sccb);
+	sccb.header.length = sizeof(sccb);
  
-	sccb.sdias_evbuf.length		  = sizeof(sdias_evbuf_t);
-	sccb.sdias_evbuf.type		  = SDIAS_OP_CODE;
+	sccb.sdias_evbuf.header.length	  = sizeof(struct sdias_evbuf);
+	sccb.sdias_evbuf.header.type	  = SDIAS_OP_CODE;
 	sccb.sdias_evbuf.event_qualifier  = EQ_SIZE;
 	sccb.sdias_evbuf.data_identifier  = DI_FCP_DUMP;
 	sccb.sdias_evbuf.event_identifier = 4712; 
 	sccb.sdias_evbuf.dbs		  = 1;
  
-	sdias_request.block = &sccb;
-	sdias_request.word  = HWC_CMDW_WRITEDATA;
+	sdias_request.sccb     = &sccb;
+	sdias_request.command  = SCLP_CMDW_WRITEDATA;
+	sdias_request.status   = SCLP_REQ_FILLED;
 	sdias_request.callback = sdias_callback;
 	debug_text_event(dump.dbf,1,"getblk2");	 
 	if(dump_sclp_send(&sdias_request) != 0){
@@ -1171,6 +1223,13 @@ sdias_get_nr_blocks(void)
 		rc = -1;
 		goto out;
 	}
+	if(sccb.header.response_code != 0x0020) {
+		DUMP_PRINT_TRACE("SCLP get nr of blocks request failed "
+				"(rc = %x)! waiting..\n",
+				(sccb.header.response_code));
+		rc = -1;
+		goto out;
+	}
 
 	debug_text_event(dump.dbf,1,"getblk4");
 	switch(sccb.sdias_evbuf.event_status){
@@ -1178,7 +1237,8 @@ sdias_get_nr_blocks(void)
 			rc = sccb.sdias_evbuf.data_block_count;
 			goto out;
 		default:
-			DUMP_PRINT_ERR("Error from SCLP. Event status = %x\n",sccb.sdias_evbuf.event_status);
+			DUMP_PRINT_ERR("Error from SCLP. Event status = %x\n",
+				sccb.sdias_evbuf.event_status);
 			rc = -1;
 			goto out;
 	}
@@ -1201,24 +1261,24 @@ out:
  *		 - < 0: ERROR - negative event status
  */
 
-static sdias_sccb_t sccb_cb __attribute__((__aligned__(4096)));
+static struct sdias_sccb sccb_cb __attribute__((__aligned__(4096)));
 static int 
 sdias_copy_blocks(void* target,int start_blk, int nr_blks)
 {
 #ifdef CONFIG_DUMP_TEST
-	memcpy_absolute(target, (start_blk-2)*4096, *nr_blks * 4096);
+	memcpy_absolute(target, (start_blk-2)*4096, nr_blks * 4096);
 	return 0;
 #else
-	hwc_request_t sdias_request = {};
+	struct sclp_req sdias_request = {};
 	int rc = 0;
 
 	memset(&sccb_cb,0,sizeof(sccb_cb));
  
-	sccb_cb.length = sizeof(sccb_cb);
+	sccb_cb.header.length  = sizeof(sccb_cb);
  
-	sccb_cb.sdias_evbuf.length		= sizeof(sdias_evbuf_t);
-	sccb_cb.sdias_evbuf.type		= SDIAS_OP_CODE;
-	sccb_cb.sdias_evbuf.flags	 	= 0;
+	sccb_cb.sdias_evbuf.header.length	= sizeof(struct sdias_evbuf);
+	sccb_cb.sdias_evbuf.header.type		= SDIAS_OP_CODE;
+	sccb_cb.sdias_evbuf.header.flags 	= 0;
 	sccb_cb.sdias_evbuf.event_qualifier	= EQ_STORE_DATA;
 	sccb_cb.sdias_evbuf.data_identifier	= DI_FCP_DUMP;
 	sccb_cb.sdias_evbuf.event_identifier	= 4712;
@@ -1235,8 +1295,9 @@ sdias_copy_blocks(void* target,int start_blk, int nr_blks)
 	sccb_cb.sdias_evbuf.dbs			= 1;
 
  
-	sdias_request.block = &sccb_cb;
-	sdias_request.word  = HWC_CMDW_WRITEDATA;
+	sdias_request.sccb     = &sccb_cb;
+	sdias_request.command  = SCLP_CMDW_WRITEDATA;
+	sdias_request.status   = SCLP_REQ_FILLED;
 	sdias_request.callback = sdias_callback;
  
 	if(dump_sclp_send(&sdias_request) != 0){
@@ -1244,19 +1305,28 @@ sdias_copy_blocks(void* target,int start_blk, int nr_blks)
 		rc = -1;
 		goto out;
 	}
+	if(sccb_cb.header.response_code != 0x0020) {
+		DUMP_PRINT_TRACE("SCLP copy blocks request failed (rc = %x)! "
+				"waiting..\n", (sccb_cb.header.response_code));
+		rc = -1;
+		goto out;
+	}
 
 	switch(sccb_cb.sdias_evbuf.event_status){
 		case EVSTATE_ALL_STORED:
-			// DUMP_PRINT_TRACE("EVSTATE_ALL_STORED:\n");
+			DUMP_PRINT_TRACE("EVSTATE_ALL_STORED:\n");
 			debug_text_event(dump.dbf, 1, "st:all");
 		case EVSTATE_PART_STORED:
-			// DUMP_PRINT_TRACE("EVSTATE_PART_STORED: %i\n",sccb_cb.sdias_evbuf.data_block_count);
+			DUMP_PRINT_TRACE("EVSTATE_PART_STORED: %i\n",
+				sccb_cb.sdias_evbuf.data_block_count);
 			debug_text_event(dump.dbf, 1, "st:part");
 			break;
 		case EVSTATE_NO_DATA:
 			DUMP_PRINT_TRACE("EVSTATE_NO_DATA\n");
 		default:
-			DUMP_PRINT_ERR("Error from SCLP while copying hsa. Event status = %x\n",sccb_cb.sdias_evbuf.event_status);
+			DUMP_PRINT_ERR("Error from SCLP while copying hsa. "
+				"Event status = %x\n",
+				sccb_cb.sdias_evbuf.event_status);
 			rc = -1;
 			goto out;
 	}
@@ -1280,7 +1350,8 @@ static int sdias_memcpy(char* target, char* src, int count)
 	if((src_ul % 4096) != 0){
 		start_blk = src_ul / 4096 + 2; 
 		if((rc = sdias_copy_blocks(sdias_memcpy_buf, start_blk,1)) < 0 ){
-			DUMP_PRINT_ERR("sdias_copy_blocks() failed: rc = %i\n",rc);
+			DUMP_PRINT_ERR("sdias_copy_blocks() failed: rc = %i\n",
+					rc);
 			goto out;
 		}
 		rel_pos = MIN((4096 - (src_ul % 4096)), count);
@@ -1292,7 +1363,8 @@ static int sdias_memcpy(char* target, char* src, int count)
 	for(;(rel_pos+4096) <= count; rel_pos += 4096){
 		start_blk = (src_ul + rel_pos) / 4096 + 2;
 		if((rc = sdias_copy_blocks(sdias_memcpy_buf, start_blk,1)) < 0 ){
-			DUMP_PRINT_ERR("sdias_copy_blocks() failed: rc = %i\n",rc);
+			DUMP_PRINT_ERR("sdias_copy_blocks() failed: rc = %i\n",
+					rc);
 			goto out;
 		}
 		memcpy(target + rel_pos, sdias_memcpy_buf, 4096);
@@ -1310,3 +1382,7 @@ static int sdias_memcpy(char* target, char* src, int count)
 out:
 	return rc;
 }
+
+late_initcall(dump_init);
+module_exit(dump_exit);
+
