@@ -1,7 +1,7 @@
 /*
  *  drivers/s390/char/dump.h
  *
- *    Copyright (C) 2003 IBM Deutschland Entwicklung GmbH, IBM Corporation
+ *    Copyright (C) 2003-2005 IBM Deutschland Entwicklung GmbH, IBM Corporation
  *    Author(s): Michael Holzheu <holzheu@de.ibm.com>
  */
 
@@ -16,5 +16,5 @@
 #define ZCORE_ERR_CPU_INFO      0x2
 #define ZCORE_ERR_OTHER         0x3
 
-void setup_dump(char* cmdline);
+void setup_dump(char* cmdline, unsigned short *init_devno);
 int  dump_init(void);
