@@ -1,7 +1,8 @@
 /******************************************************************************/
 /*  zfcp_dumper.c                                                             */
 /*                                                                            */
-/*    Copyright (C) 2003 IBM Deutschland Entwicklung GmbH, IBM Corporation    */
+/*    Copyright (C) 2003-2005 IBM Deutschland Entwicklung GmbH,               */
+/*                  IBM Corporation                                           */
 /*    Author(s): Michael Holzheu <holzheu@de.ibm.com>                         */
 /*               Andreas Herrmann <aherrman@de.ibm.com>                       */
 /******************************************************************************/
@@ -11,6 +12,7 @@
 #include <sys/ioctl.h>
 #include <sys/time.h>
 #include <sys/reboot.h>
+#include <sys/mount.h>
 #include <linux/reboot.h>
 #include <fcntl.h>
 #include <stdlib.h>
@@ -42,13 +44,14 @@
 
 #define PROC_CMDLINE     "/proc/cmdline"
 #define DEV_ZCORE        "/dev/zcore"
-#define PROC_FCPMAP      "/proc/scsi/zfcp/add_map"
-#define PROC_SCSI_SCSI   "/proc/scsi/scsi"
-#define DUMP_DISK_PATH   "/dev/scsi/host0/bus0/target1/lun0"
+#define DEV_ZCORE_MAJOR  200
+#define DEV_ZCORE_MINOR  0
+#define DEV_SCSI         "/dev/scsidev"
+#define DEV_SCSI_MAJOR   8
+#define DEV_NULL         "/dev/null"
+#define DEV_NULL_MAJOR   1
+#define DEV_NULL_MINOR   3 
 #define DUMP_MOUNT_POINT "/mnt"
-#define CMD_E2FSCK       "/sbin/e2fsck -y"
-#define CMD_MOUNT        "/bin/mount"
-#define CMD_UMOUNT       "/bin/umount"
 
 #define PARM_DUMP_DIR            "dump_dir"
 #define PARM_DUMP_DIR_DFLT       "/"
@@ -62,13 +65,16 @@
 #define PARM_DUMP_COMPRESS_DFLT   PARM_DUMP_COMPRESS_NONE
 
 #define PARM_DUMP_MEM            "dump_mem"
+#ifdef __s390x__
 #define PARM_DUMP_MEM_DFLT       0xffffffffffffffff
+#else
+#define PARM_DUMP_MEM_DFLT       0xffffffff
+#endif
 
 #define PARM_DUMP_DEBUG          "dump_debug"
 #define PARM_DUMP_DEBUG_DFLT      2
 #define PARM_DUMP_DEBUG_MIN       1
 #define PARM_DUMP_DEBUG_MAX       6
-#define PARM_DUMP_DEBUG_INTERACTIVE 4712
 
 #define PARM_DUMP_MODE               "dump_mode"
 #define PARM_DUMP_MODE_INTERACT      "interactive"
@@ -83,8 +89,8 @@
 
 #define ONE_MB 1048576
 
-#define SLEEP_TIME_ERASE 5 /* 5 sec */
-#define SLEEP_TIME_END   3 /* 3 sec */
+#define SLEEP_TIME_ERASE 5 /* seconds */
+#define SLEEP_TIME_END   3 /* seconds */
 
 /******************************************************************************/
 /* defines dump format                                                        */
@@ -101,7 +107,6 @@
  *      2. the possibly compressed page headers and data
  */
 #define DUMP_BUFFER_SIZE     (64 * 1024)  /* size of dump buffer (0x10000) */
-#define DUMP_HEADER_SIZE     DUMP_BUFFER_SIZE /* why we need this ?? */
  
 /* header definitions for dumps from s390 standalone dump tools */
 #define DUMP_MAGIC_S390SA     0xa8190173618f23fdULL /* s390sa magic number */
@@ -112,14 +117,6 @@
 #define DUMP_VERSION_NUMBER 0x8      /* dump version number             */
 #define DUMP_PANIC_LEN      0x100    /* dump panic string length        */
  
-/* dump levels - type specific stuff added later -- add as necessary */
-#define DUMP_LEVEL_NONE        0x0   /* no dumping at all -- just bail   */
-#define DUMP_LEVEL_HEADER      0x1   /* kernel dump header only          */
-#define DUMP_LEVEL_KERN        0x2   /* dump header and kernel pages     */
-#define DUMP_LEVEL_USED        0x4   /* dump header, kernel/user pages   */
-#define DUMP_LEVEL_ALL_RAM     0x8   /* dump header, all RAM pages       */
-#define DUMP_LEVEL_ALL         0x10  /* dump all memory RAM and firmware */
- 
 /* dump compression options -- add as necessary */
 #define DUMP_COMPRESS_NONE     0x0   /* don't compress this dump      */
 #define DUMP_COMPRESS_RLE      0x1   /* use RLE compression           */
@@ -220,7 +217,8 @@ typedef struct dump_page_s {
 
 
 /* Compression function */
-typedef int (*compress_fn_t)(const char *old, uint32_t old_size, char *new, uint32_t size);
+typedef int (*compress_fn_t)(const unsigned char *old, uint32_t old_size,
+		unsigned char *new, uint32_t size);
 
 
 /******************************************************************************/
@@ -230,6 +228,7 @@ typedef int (*compress_fn_t)(const char *old, uint32_t old_size, char *new, uint
 static void release_hsa(void);
 static int parse_parmline(void);
 static int dump_init_sig(void);
+static int dump_tune_vm(void);
 static void dump_terminate(void);
 static int dump_create_s390sa(char* sourcedev, char* dumpdir);
 static void dump_display_progress(uint64_t mb_written, uint64_t mb_max);
@@ -272,11 +271,11 @@ static int parse_parmline(void){
 
 	fh = open(PROC_CMDLINE,O_RDONLY);
 	if(fh == -1){
-                PRINT_PERR("open "PROC_CMDLINE" failed");
+                PRINT_PERR("open %s failed\n",PROC_CMDLINE);
                 rc = -1;goto out;
         }
 	if((count = read(fh,g.parmline,CMDLINE_MAX_LEN)) == -1){
-		PRINT_PERR("read "PROC_CMDLINE" failed");
+		PRINT_PERR("read %s failed\n", PROC_CMDLINE);
 		rc = -1;goto out;
 	}
 	g.parmline[count-1]='\0'; /* remove \n */
@@ -287,8 +286,10 @@ static int parse_parmline(void){
                 token = strtok(NULL," \t\n");
                 i++;
 		if(i >= PARM_MAX){
-			PRINT_WARN("THIS SHOULD NOT HAPPEN: More than %i kernel parmameters specified\n",PARM_MAX);
-			PRINT_WARN("                        The rest of the parameters will not be considered!\n");
+			PRINT_WARN("THIS SHOULD NOT HAPPEN: More than %i "
+				"kernel parmameters specified\n",PARM_MAX);
+			PRINT_WARN("                        The rest of the "
+				"parameters will not be considered!\n");
 			break;
 		}
         }
@@ -303,8 +304,10 @@ static int parse_parmline(void){
                 } else if(strcmp(token, PARM_DUMP_DIR) == 0){
                         g.parm_dump_dir = strtok(NULL,"=");
 			if(g.parm_dump_dir == NULL){
-				PRINT_WARN("No value for '%s' parameter specified\n", PARM_DUMP_DIR);
-				PRINT_WARN("Using default: %s\n",PARM_DUMP_DIR_DFLT);
+				PRINT_WARN("No value for '%s' parameter "
+					"specified\n", PARM_DUMP_DIR);
+				PRINT_WARN("Using default: %s\n",
+					PARM_DUMP_DIR_DFLT);
 				g.parm_dump_dir = PARM_DUMP_DIR_DFLT;
 			}
 
@@ -313,7 +316,8 @@ static int parse_parmline(void){
                 } else if(strcmp(token, PARM_DUMP_PART) == 0){
                         g.parm_dump_part = strtok(NULL,"=");
 			if(g.parm_dump_part == NULL){
-				PRINT_ERR("No value for '%s' parameter specified\n", PARM_DUMP_PART);
+				PRINT_ERR("No value for '%s' parameter "
+					"specified\n", PARM_DUMP_PART);
 				rc = -1;
 				goto out;
 			}
@@ -323,7 +327,8 @@ static int parse_parmline(void){
                 } else if(strcmp(token, PARM_DUMP_MEM) == 0){
 			char* mem_str = strtok(NULL,"=");
 			if(mem_str == NULL){
-				PRINT_ERR("No value for '%s' parameter specified\n", PARM_DUMP_MEM);
+				PRINT_ERR("No value for '%s' parameter "
+					"specified\n", PARM_DUMP_MEM);
 				rc = -1;
 				goto out;
 			}
@@ -334,13 +339,19 @@ static int parse_parmline(void){
                 } else if(strcmp(token, PARM_DUMP_COMPRESS) == 0){
 			g.parm_dump_compress = strtok(NULL,"=");
 			if(g.parm_dump_compress == NULL){
-				PRINT_WARN("No value for '%s' parameter specified\n", PARM_DUMP_COMPRESS);
-				PRINT_WARN("Using default: %s\n", PARM_DUMP_COMPRESS_DFLT);
+				PRINT_WARN("No value for '%s' parameter "
+					"specified\n", PARM_DUMP_COMPRESS);
+				PRINT_WARN("Using default: %s\n",
+					PARM_DUMP_COMPRESS_DFLT);
 				g.parm_dump_compress = PARM_DUMP_COMPRESS_DFLT;
-			} else if((strcmp(g.parm_dump_compress, PARM_DUMP_COMPRESS_GZIP) != 0) &&
-           		          (strcmp(g.parm_dump_compress, PARM_DUMP_COMPRESS_NONE) != 0)){
-				PRINT_WARN("Unknown dump compression '%s' specified!\n",g.parm_dump_compress);
-				PRINT_WARN("Using default: %s\n",PARM_DUMP_COMPRESS_DFLT);
+			} else if((strcmp(g.parm_dump_compress,
+					PARM_DUMP_COMPRESS_GZIP) != 0) &&
+           		          (strcmp(g.parm_dump_compress,
+					PARM_DUMP_COMPRESS_NONE) != 0)){
+				PRINT_WARN("Unknown dump compression '%s' "
+					"specified!\n",g.parm_dump_compress);
+				PRINT_WARN("Using default: %s\n",
+					PARM_DUMP_COMPRESS_DFLT);
 				g.parm_dump_compress = PARM_DUMP_COMPRESS_DFLT;
 			}
 
@@ -349,19 +360,23 @@ static int parse_parmline(void){
 		} else if(strcmp(token, PARM_DUMP_DEBUG) == 0){
 			char *s = strtok(NULL,"=");
 			if(s == NULL){
-				PRINT_WARN("No value for '%s' parameter specified\n", PARM_DUMP_DEBUG);
-				PRINT_WARN("Using default: %d\n",PARM_DUMP_DEBUG_DFLT);
+				PRINT_WARN("No value for '%s' parameter "
+					"specified\n", PARM_DUMP_DEBUG);
+				PRINT_WARN("Using default: %d\n",
+					PARM_DUMP_DEBUG_DFLT);
 
 			} else {
 				g.parm_dump_debug = atoi(s);
-				if(g.parm_dump_debug == PARM_DUMP_DEBUG_INTERACTIVE){
-					char* args[] = {"/bin/sh" , NULL};
-					execvp("/bin/sh",args);
-				}
-				if((g.parm_dump_debug < PARM_DUMP_DEBUG_MIN) || (g.parm_dump_debug > PARM_DUMP_DEBUG_MAX)){
-					PRINT_WARN("Invalid value (%i) for %s parameter specified (allowed range is %i - %i)\n",g.parm_dump_debug, PARM_DUMP_DEBUG, PARM_DUMP_DEBUG_MIN, PARM_DUMP_DEBUG_MAX);
-					PRINT_WARN("Using default: %i\n", PARM_DUMP_DEBUG_DFLT);
-					g.parm_dump_debug = PARM_DUMP_DEBUG_DFLT;
+				if((g.parm_dump_debug < PARM_DUMP_DEBUG_MIN)
+				|| (g.parm_dump_debug > PARM_DUMP_DEBUG_MAX)){
+					PRINT_WARN("Invalid value (%i) for %s "
+					"parameter specified (allowed range is "
+					"%i - %i)\n",g.parm_dump_debug,
+					PARM_DUMP_DEBUG, PARM_DUMP_DEBUG_MIN,
+					PARM_DUMP_DEBUG_MAX);
+					PRINT_WARN("Using default: %i\n",
+					PARM_DUMP_DEBUG_DFLT);
+					g.parm_dump_debug=PARM_DUMP_DEBUG_DFLT;
 				}
 			}
 
@@ -370,15 +385,18 @@ static int parse_parmline(void){
 		} else if(strcmp(token, PARM_DUMP_MODE) == 0){
 			char *s = strtok(NULL,"=");
 			if(s == NULL){
-				PRINT_WARN("No value for '%s' parameter specified\n", PARM_DUMP_MODE);
-				PRINT_WARN("Using default: %s\n",PARM_DUMP_MODE_DFLT);
+				PRINT_WARN("No value for '%s' parameter "
+					"specified\n", PARM_DUMP_MODE);
+				PRINT_WARN("Using default: %s\n",
+					PARM_DUMP_MODE_DFLT);
 			} else if(strcmp(s,PARM_DUMP_MODE_INTERACT) == 0){
 				g.parm_dump_mode = PARM_DUMP_MODE_INTERACT_NUM;
 			} else if(strcmp(s,PARM_DUMP_MODE_AUTO) == 0){
 				g.parm_dump_mode = PARM_DUMP_MODE_AUTO_NUM;
 			} else {
 				PRINT_WARN("Unknown dump mode: %s\n",s);
-				PRINT_WARN("Using default: %s\n",PARM_DUMP_MODE_DFLT);
+				PRINT_WARN("Using default: %s\n",
+					PARM_DUMP_MODE_DFLT);
 			}
 		}
         }
@@ -394,8 +412,7 @@ static int parse_parmline(void){
 		PRINT_TRACE("dump mode : %s\n",PARM_DUMP_MODE_INTERACT);
 	}
 
-	sprintf(g.dump_dev, DUMP_DISK_PATH"/part%s",g.parm_dump_part);
-	sprintf(g.dump_dir, DUMP_MOUNT_POINT"/%s",g.parm_dump_dir);
+	sprintf(g.dump_dir, "%s/%s",DUMP_MOUNT_POINT, g.parm_dump_dir);
 out:
 	if(fh != -1)
 		close(fh);
@@ -417,11 +434,11 @@ int dump_check_zcore(void)
 
 	fh = open(DEV_ZCORE,O_RDWR); 
 	if(fh == -1){
-		PRINT_PERR("open "DEV_ZCORE" failed");
+		PRINT_PERR("open %s failed\n", DEV_ZCORE);
 		rc = -1; goto out;
 	}
 	if(ioctl(fh, ZCORE_IOCTL_GET_ERROR, &error_code) == -1){
-		PRINT_PERR("ioctl 'get error code' failed");
+		PRINT_PERR("ioctl 'get error code' failed\n");
 		rc = -1; goto out;
 	}
 	switch(error_code){
@@ -459,7 +476,7 @@ char* get_xml_string(void)
 
 	fh = open(DEV_ZCORE,O_RDWR); 
 	if(fh == -1){
-		PRINT_PERR("open "DEV_ZCORE" failed");
+		PRINT_PERR("open %s failed", DEV_ZCORE);
 		goto out;
 	}
 	if(ioctl(fh, ZCORE_IOCTL_XMLSIZE, &xml_len) == -1){
@@ -497,7 +514,7 @@ int get_hsa_size(void)
 
 	fh = open(DEV_ZCORE,O_RDWR); 
 	if(fh == -1){
-		PRINT_PERR("open "DEV_ZCORE" failed");
+		PRINT_PERR("open %s failed", DEV_ZCORE);
 		rc = -1; goto out;
 	}
 	if(ioctl(fh, ZCORE_IOCTL_HSASIZE, &rc) == -1){
@@ -510,6 +527,27 @@ out:
 		close(fh);
 	return rc;
 }
+
+static int
+write_to_file(const char* fcp_file, const char* command)
+{
+	int rc = 0, fh = 0;
+
+        fh = open(fcp_file,O_WRONLY);
+        if(fh == -1){
+                PRINT_PERR("Could not open %s",fcp_file);
+                rc = -1; goto out;
+        }
+	if(write(fh, command, strlen(command)) != strlen(command)){
+		PRINT_PERR("Write to %s failed",fcp_file);
+		rc = -1; goto out;
+	};
+out:
+	if(fh)
+		close(fh);
+	return rc;
+}
+
 /*
  * enable_zfcp_device()
  *
@@ -522,50 +560,28 @@ int enable_zfcp_device(fcp_info_t* fcp_info)
 {
 	int fh = -1, fh_check;
 	int rc = 0;
-	char fcp_enable_string[1024] = {0};
+	char command_string[1024] = {0};
+	char fcp_file[1024] = {0};
 
-	sprintf(fcp_enable_string, "0x%x 0x00000001:0x%016llx 0x00000000:0x%016llx\n",fcp_info->devno,fcp_info->wwpn, fcp_info->fcp_lun);
-	PRINT_TRACE("enable: %s\n",fcp_enable_string);
-        fh = open(PROC_FCPMAP,O_RDWR);
-        if(fh == -1){
-                PRINT_PERR("Could not open "PROC_FCPMAP);
-                rc = -1; goto out;
-        }
-	if(write(fh, fcp_enable_string, strlen(fcp_enable_string)) != strlen(fcp_enable_string)){
-		PRINT_PERR("Write to "PROC_FCPMAP" failed");
-		rc = -1; goto out;
-	};
-	close(fh);
-
-	memset(fcp_enable_string, 0 , sizeof(fcp_enable_string));
-
-        fh = open(PROC_SCSI_SCSI,O_RDWR);
-        if(fh == -1){
-                PRINT_PERR("Could not open "PROC_SCSI_SCSI);
-                rc = -1; goto out;
-        }
-	sprintf(fcp_enable_string,  "scsi add-single-device 0 0 1 0\n");
-	if(write(fh, fcp_enable_string, strlen(fcp_enable_string)) != strlen(fcp_enable_string)){
-		PRINT_PERR("Write to "PROC_SCSI_SCSI" failed");
-		rc = -1; goto out;
-	};
-
-	/* Check if dump device is available */
-
-	if((fh_check = open(g.dump_dev,O_RDONLY)) == -1){
-		PRINT_ERR("No such file: %s\n",g.dump_dev);
-		rc = -1; goto out;
-	} else {
-		close(fh_check);
-	}
-
-out:
-	if(fh != -1)
-		close(fh);
-	return rc;
+	sprintf(command_string, "1\n");
+	sprintf(fcp_file,"/sys/bus/ccw/drivers/zfcp/0.0.%04x/online",
+		fcp_info->devno);
+	PRINT_TRACE("online: %s %s\n",command_string,fcp_file);
+	if(write_to_file(fcp_file, command_string))
+		return -1;
+	sprintf(command_string, "0x%016llx\n",fcp_info->wwpn);
+	sprintf(fcp_file,"/sys/bus/ccw/drivers/zfcp/0.0.%04x/port_add",
+		fcp_info->devno);
+	if(write_to_file(fcp_file, command_string))
+		return -1;
+	sprintf(command_string, "0x%016llx\n",fcp_info->fcp_lun);
+	sprintf(fcp_file,"/sys/bus/ccw/drivers/zfcp/0.0.%04x/0x%016llx/"
+		"unit_add",fcp_info->devno,fcp_info->wwpn);
+	if(write_to_file(fcp_file, command_string))
+		return -1;
+	return 0;
 }
 
-
 /*
  * mount_dump_device()
  *
@@ -575,17 +591,29 @@ out:
  */
 int mount_dump_device(void)
 {
-	int rc = 0;
+	int rc = 0, pid;
 
-	sprintf(g.cmd, CMD_E2FSCK" %s", g.dump_dev);
-	PRINT_TRACE("command: %s\n",g.cmd);
-	system(g.cmd); // We do not check the return code here since e2fsck returns 
-	               // also a bad return code in case the filesystem has been repaired
-	sprintf(g.cmd, CMD_MOUNT" %s "DUMP_MOUNT_POINT"",g.dump_dev);
-	PRINT_TRACE("command: %s\n",g.cmd);
-        if(system(g.cmd) != 0){
-                PRINT_ERR("mount failed");
-                rc = -1; goto out;
+	/* e2fsck */
+	PRINT_TRACE("e2fsck\n");
+	pid = fork();
+	if(pid < 0){
+		PRINT_PERR("fork failed\n");
+		rc = -1; goto out;
+	} else if(pid == 0){
+		execl("/sbin/e2fsck", "e2fsck", DEV_SCSI, "-y", NULL);
+	} else {
+		waitpid (pid, NULL, 0);
+	}
+
+	/* mount dump device */	
+	PRINT_TRACE("mount\n");
+	if(mount(DEV_SCSI, DUMP_MOUNT_POINT, "ext3", 0, NULL) == 0){
+		rc = 0;
+		goto out;
+	}
+	if(mount(DEV_SCSI, DUMP_MOUNT_POINT, "ext2", 0, NULL) != 0){
+		PRINT_PERR("mount failed\n");
+		rc = -1; goto out;
         }
 out:
 	return rc;
@@ -601,9 +629,7 @@ out:
 int umount_dump_device(void)
 {
 	int rc = 0;
-	sprintf(g.cmd, CMD_UMOUNT" %s ",g.dump_dev);
-	PRINT_TRACE("command: %s\n",g.cmd);
-        if(system(g.cmd) != 0){
+        if(umount(DUMP_MOUNT_POINT) != 0){
                 PRINT_PERR("umount failed");
                 rc = -1; goto out;
         }
@@ -629,7 +655,7 @@ static void release_hsa(void)
 	fflush(stderr);
 	fh = open(DEV_ZCORE,O_RDWR);
         if(fh == -1){
-		PRINT_PERR("open "DEV_ZCORE" failed");
+		PRINT_PERR("open %s failed", DEV_ZCORE);
 		goto out;
 	}
         if(ioctl(fh, ZCORE_IOCTL_RELEASE_HSA,NULL) == -1){
@@ -643,58 +669,6 @@ out:
 	return;
 }
 
-/*
- * print_addr()
- *
- * Print buffer to screen. Only used for debugging purposes
- * Parameter: start - Start address
- *            size  - length
- * Return:   0 - ok
- *         !=0 - error
- */
-static int print_addr(unsigned long start, unsigned long size){
-	int fh;
-	char* buf;
-	unsigned long i;
-	unsigned long* ptr;
-	int rc = 0;
-
-	size += size - (size - (size % 16));
-	fh = open(DEV_ZCORE,O_RDWR);
-	if(fh == -1){
-		PRINT_ERR("Open failed\n");
-		rc = -1; goto out;
-	}
-	lseek(fh, start, SEEK_SET);
-	buf = malloc(size);
-	if(buf == NULL){
-		PRINT_ERR("Malloc failed\n");
-		rc = -1; goto out;
-	}
-	if(read(fh, buf, size) != size){
-		PRINT_ERR("read failed\n");
-		rc = -1; goto out;
-	}
-	for ( i = 0; i < size; i+= 16) {
-		int j;
-		ptr = (unsigned long*)&buf[i];
-		printf("%016lx: %016lx %016lx | ", i+start , ptr[0],ptr[1]);
-		for(j = 0; j < 16; j++) {
-			if(isprint(((char*)ptr)[j])){
-				printf("%c",((char*)ptr)[j]);
-			} else {
-				printf(".");
-			}
-		}
-		printf("\n");
-	}	
-out:
-	if(fh != -1)
-		close(fh);
-	return rc;
-}
-
-
 /*
  * main()
  *
@@ -711,49 +685,57 @@ int main(int argc, char* argv[])
 	PRINT("Version %s (64 bit)\n",RELEASE_STRING);
 #else
 	PRINT("Linux for zSeries System Dumper starting\n");
-	PRINT("Version %s (31 bit)\n",RELEASE_STRING);
+	PRINT("Version %s (32 bit)\n",RELEASE_STRING);
 #endif
-
-	/* Interactive release HSA: Only for debugging */
-
-	if(argc == 2){
-		if(strcmp(argv[1],"release") == 0){
-			printf("releasing hsa\n");
-			release_hsa();
-		} else {
-			printf("unknown parameter: %s\n",argv[1]);
-		}
-		exit(0);
-	}
-
-	/* Print Memory content to screen: Only for debugging */
-
-	if(argc == 3){
-		int rc;
-		rc = print_addr(atol(argv[1]),atol(argv[2]) );
-		exit(rc);
-	}
-		
-	if(dump_init_sig() != 0){
+	if(dump_init_sig()){
 		PRINT_ERR("Init Signals failed!\n");
 		rc = 1; goto out;
 	}
-
-	if(parse_parmline() != 0){
+        if(mount("proc","/proc","proc", 0, NULL)){
+                PRINT_PERR("Unable to mount proc\n");
+		rc = 1; goto out;
+	}
+	if(dump_tune_vm()){
+		PRINT_PERR("Unable to set VM settings\n");
+		rc = 1; goto out;
+	}
+        if(mount("sysfs","/sys","sysfs", 0, NULL)){
+                PRINT_PERR("Unable to mount sysfs\n");
+		rc = 1; goto out;
+	}
+	if(parse_parmline()){
 		PRINT_ERR("Could not parse parmline\n");
 		rc = 1; goto out;
 	}
+	if(mknod(DEV_NULL, S_IFCHR | 0600,
+		makedev(DEV_NULL_MAJOR, DEV_NULL_MINOR))){
+		rc = 1; goto out;
+	}
+	if(mknod(DEV_ZCORE, S_IFCHR | 0600,
+		makedev(DEV_ZCORE_MAJOR, DEV_ZCORE_MINOR))){
+		rc = 1; goto out;
+	}
+	if(mknod(DEV_SCSI, S_IFBLK | 0600,
+		makedev(DEV_SCSI_MAJOR, atoi(g.parm_dump_part)))){
+		rc = 1; goto out;
+	}
+	if(dump_check_zcore()){
+		PRINT_ERR("Check if your Microcode is up to date!\n");
+		rc = 1; goto out;
+	}
 	xml_string = get_xml_string();
 	if(xml_string == NULL){
 		PRINT_ERR("Could not get XML data\n");
 		rc = 1; goto out;
 	}
-	if(get_fcp_info(xml_string,&fcp_info) != 0){
+	if(get_fcp_info(xml_string,&fcp_info)){
 		PRINT_ERR("Could not parse XML data:\n");
 		PRINT_ERR("%s\n",xml_string);
 		rc = 1; goto out;
 	}
 
+	PRINT(" \n"); /* leading blank is needed that sclp console prints */
+                      /* the newline */
 	PRINT("DUMP PARAMETERS:\n");
 	PRINT("================\n");
 	PRINT("devno    : %x\n",fcp_info.devno);
@@ -765,42 +747,37 @@ int main(int argc, char* argv[])
 	PRINT("directory: %s\n",g.parm_dump_dir);
 	PRINT("compress : %s\n",g.parm_dump_compress);
 
-	if(strcmp(fcp_info.type,"dump") != 0){
+	if(strcmp(fcp_info.type,"dump")){
 		PRINT_ERR("IPL type is not dump\n");
 		rc = 1; goto out;
 	}
-
-
-	if(dump_check_zcore() != 0){
-		PRINT_ERR("Check if your Microcode is up to date!\n");
-		rc = 1; goto out;
-	}
-
 	if((g.hsa_size = get_hsa_size()) < 0){
 		PRINT_ERR("Could not determine HSA size!\n");
 		rc = 1; goto out;
 	}
+	PRINT(" \n");
 	PRINT("MOUNT DUMP PARTITION:\n");
 	PRINT("=====================\n");
-	if(enable_zfcp_device(&fcp_info) != 0){
+	if(enable_zfcp_device(&fcp_info)){
 		PRINT_ERR("Could not enable dump device\n");
-		rc=1; goto out;
+		rc = 1; goto out;
 	}
-	if(mount_dump_device() != 0){
+	if(mount_dump_device()){
 		PRINT_ERR("Could not mount dump device\n");
-		rc=1; goto out;
+		rc = 1; goto out;
 	}
+	PRINT(" \n");
 	PRINT("DUMP PROCESS STARTED:\n");
 	PRINT("=====================\n");
-	if(dump_create_s390sa(DEV_ZCORE, g.dump_dir) != 0){
-		rc=1; // no goto out -> unmount device first!
+	if(dump_create_s390sa(DEV_ZCORE, g.dump_dir)){
+		rc = 1; /* no goto out -> unmount device first! */
 	}
-
-	if(umount_dump_device() != 0){
+	if(umount_dump_device()){
 		PRINT_ERR("Could not umount dump device\n");
-		rc=1; goto out;
+		rc = 1; goto out;
 	}
 out:
+	PRINT(" \n");
 	if(rc==0){
 		PRINT("DUMP 'dump.%i' COMPLETE\n",g.dump_nr);
 	} else {
@@ -878,6 +855,46 @@ out:
 	return rc;
 }
 
+/*
+ * dump_tune_vm()
+ *
+ * Set memory management parameters: Ensure that dirty pages are written
+ * early enough! See "Documentation/filesystems/proc.txt"
+ * Return:   0 - ok
+ *         !=0 - error
+ */
+static int
+dump_tune_vm(void)
+{
+	char* sysctl_names[] = {"/proc/sys/vm/dirty_ratio",
+				"/proc/sys/vm/dirty_background_ratio",
+				"/proc/sys/vm/dirty_writeback_centisecs",
+				"/proc/sys/vm/dirty_expire_centisecs",
+				"/proc/sys/vm/vfs_cache_pressure",
+				"/proc/sys/vm/lowmem_reserve_ratio",
+				NULL};
+	char* sysctl_values[] = {"2","5","50","50","500","32",NULL};
+	int fh, i=0, rc=0;
+
+	while(sysctl_names[i]){
+		fh = open(sysctl_names[i],O_RDWR);
+		if(fh < 0){
+			rc = -1;
+			goto out;
+		}
+		if(write(fh, sysctl_values[i], strlen(sysctl_values[i])) !=
+			strlen(sysctl_values[i])){
+			rc = -1;
+			goto out;
+		}
+		i++;
+	}
+out:
+	if(fh >= 0)
+		close(fh);
+	return rc;
+}
+
 /*
  * dump_get_num()
  *
@@ -899,7 +916,7 @@ dump_get_num(char *dumpdir, int mode)
 
 	dir=opendir(dumpdir);
 	if(!dir){
-		PRINT_PERR("Cannot evalute dump number");
+		PRINT_PERR("Cannot evalute dump number\n");
 		rc = -2; goto out;
 	}
 
@@ -918,8 +935,8 @@ dump_get_num(char *dumpdir, int mode)
 				continue;
 			}
 			if(num < 0) {
-				// XXX remove?
-				/* In the unlikely case of 'dump.-1', 'dump.-10' etc */
+				/* In the unlikely case of 'dump.-1',
+				   'dump.-10' etc */
 				continue;
 			} if(!dump_found){
 				dump_found = 1;
@@ -929,7 +946,8 @@ dump_get_num(char *dumpdir, int mode)
 			} else if(mode == DUMP_FIRST) {
 				rc = MIN(num,rc);
 			} else {
-				PRINT_ERR("BUG in %s:%d (unknown mode: %d)\n",__FILE__,__LINE__,mode);
+				PRINT_ERR("BUG in %s:%d (unknown mode: %d)\n",
+					__FILE__,__LINE__,mode);
 				rc = -3;
 				goto out;
 			}
@@ -961,7 +979,8 @@ erase_oldest_dump(void)
 
 	dump_nr = dump_get_num(g.dump_dir,DUMP_FIRST);
 	if(dump_nr < 0){
-		PRINT_ERR("BUG: cannot delete dump since number cannot be evaluated\n");
+		PRINT_ERR("BUG: cannot delete dump since number cannot be "
+			"evaluated\n");
 		rc = -1; goto out;
 	}
 	if(dump_nr == g.dump_nr){
@@ -969,7 +988,7 @@ erase_oldest_dump(void)
 		PRINT_ERR("No space left on device!\n");
 		rc = -1; goto out;
 	}
-	sprintf(dname,"%s/dump.%i",g.dump_dir,dump_nr);
+	sprintf(dname,"dump.%i",dump_nr);
 	PRINT("No more space left on device!\n");
 
 	if(g.parm_dump_mode==PARM_DUMP_MODE_AUTO_NUM){
@@ -983,15 +1002,16 @@ erase_oldest_dump(void)
 			rc = -1; goto out;
 		}
 	}
-	PRINT("OK! Removing dump: '%s' ...\n",dname);
+	sprintf(dname, "%s/dump.%i",g.dump_dir,dump_nr);
 	if(unlink(dname) == -1){
 		PRINT_PERR("Could not remove dump\n");
 		rc = -1; goto out;
 	}
 	sync();
-	sleep(SLEEP_TIME_ERASE); // XXX EXT3DEFINE /* Hack in order to give ext3 time to discover that there */
-	          /* is space avavailable again */
-	PRINT("Dump '%s' removed\n",dname);
+	/* Wait in order to give ext3 time to discover that file has been */
+        /* removed.  */
+	sleep(SLEEP_TIME_ERASE); 
+	PRINT("Dump removed!\n");
 out:
 	return rc;
 }
@@ -1045,7 +1065,8 @@ out:
  */
  
 int
-compress_gzip(const char *old, uint32_t old_size, char *new, uint32_t new_size)
+compress_gzip(const unsigned char *old, uint32_t old_size, unsigned char *new,
+		uint32_t new_size)
 {
         int rc;
         unsigned long len = old_size;
@@ -1059,7 +1080,8 @@ compress_gzip(const char *old, uint32_t old_size, char *new, uint32_t new_size)
                         rc = -1;
                         break;
                 case Z_BUF_ERROR:
-                        /* In this case the compressed output is bigger than the uncompressed */
+                        /* In this case the compressed output is bigger than
+			   the uncompressed */
                         rc = -1;
                         break;
                 case Z_DATA_ERROR:
@@ -1080,7 +1102,8 @@ compress_gzip(const char *old, uint32_t old_size, char *new, uint32_t new_size)
  * Do nothing! - No compression
  */
 int
-compress_none(const char *old, uint32_t old_size, char *new, uint32_t new_size)
+compress_none(const unsigned char *old, uint32_t old_size, unsigned char *new,
+	uint32_t new_size)
 {
         return -1;
 }
@@ -1109,10 +1132,10 @@ s390sa_to_reg_header(dump_header_s390sa_t* dh_s390sa, dump_header_t* dh)
         dh->memory_end     = dh_s390sa->memory_end;
         dh->num_dump_pages = dh_s390sa->num_pages;
         dh->page_size      = dh_s390sa->page_size;
-        dh->dump_compress  = DUMP_COMPRESS_GZIP;
         dh->dump_level     = dh_s390sa->dump_level;
  
-        sprintf(dh->panic_string,"zSeries-dump (CPUID = %16"FMT64"x)",dh_s390sa->cpu_id);
+        sprintf(dh->panic_string,"zSeries-dump (CPUID = %16"FMT64"x)",
+		dh_s390sa->cpu_id);
  
         if(dh_s390sa->arch_id == DH_ARCH_ID_S390){
                 strcpy(dh->utsname_machine,"s390");
@@ -1145,11 +1168,13 @@ dump_display_progress(uint64_t bytes_written, uint64_t bytes_max)
 
 	gettimeofday(&t, NULL);
 	time = t.tv_sec;
-	if ((time < g.last_progress_time) && (bytes_written != bytes_max) && (bytes_written != 0))
+	if ((time < g.last_progress_time) && (bytes_written != bytes_max)
+		&& (bytes_written != 0))
 		return;
 	g.last_progress_time = time + 10;
 	percent_written = ((double) bytes_written / (double) bytes_max) * 100.0;
-	PRINT(" %4i MB of %4i MB (%5.1f%% )\n", (int)(bytes_written/ONE_MB), (int)(bytes_max/ONE_MB), percent_written);
+	PRINT(" %4i MB of %4i MB (%5.1f%% )\n", (int)(bytes_written/ONE_MB),
+		(int)(bytes_max/ONE_MB), percent_written);
 	fflush(stdout);
 }
 
@@ -1182,7 +1207,8 @@ dump_create_s390sa(char* sourcedev, char* dumpdir)
 		PRINT_ERR("Specified dump dir '%s' not found!\n",dumpdir);
 		rc = -1; goto out;
 	} else if(!S_ISDIR(stat_buf.st_mode)){
-		PRINT_ERR("Specified dump dir '%s' is not a directory!\n", dumpdir);
+		PRINT_ERR("Specified dump dir '%s' is not a directory!\n",
+			dumpdir);
 		rc = -1; goto out;
 	}
 	
@@ -1205,33 +1231,39 @@ dump_create_s390sa(char* sourcedev, char* dumpdir)
 
         /* make the new filename */
         sprintf(dump_file_name, "%s/dump.%d", dumpdir, g.dump_nr);
-        if ((fp_dump = open(dump_file_name, O_CREAT|O_RDWR|O_TRUNC, (S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH))) < 0) {
-                        PRINT_ERR("open() of dump file \"%s\" failed!\n", dump_file_name);
+        if ((fp_dump = open(dump_file_name, O_CREAT|O_RDWR|O_TRUNC,
+		(S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH))) < 0) {
+                        PRINT_ERR("open() of dump file \"%s\" failed!\n",
+				dump_file_name);
 			rc = -1; goto out;
         }
 
 	PRINT("dump file: dump.%d\n",g.dump_nr);
-
-	if(strcmp(g.parm_dump_compress,PARM_DUMP_COMPRESS_GZIP) == 0)
-		compress_fn = compress_gzip;
-	else
-		compress_fn = compress_none;
-
         memset(&dh, 0, sizeof(dh));
 
-	/* get the dump header
-         */
+	/* get the dump header */
 
         if (lseek(fp_src, 0, SEEK_SET) < 0) {
-                PRINT_ERR("Cannot lseek() to get the dump header from the dump file!\n");
+                PRINT_ERR("Cannot lseek() to get the dump header from the "
+			"dump file!\n");
                 rc = -1; goto out;
         }
-        if (read(fp_src, (char *)&s390_dh, sizeof(s390_dh)) != sizeof(s390_dh)) {
+        if (read(fp_src,(char *)&s390_dh, sizeof(s390_dh)) != sizeof(s390_dh)) {
 		PRINT_ERR("Cannot read() dump header from dump file!\n");
                 rc = -1; goto out;
         }
+
         s390sa_to_reg_header(&s390_dh,&dh);
 
+	if(strcmp(g.parm_dump_compress,PARM_DUMP_COMPRESS_GZIP) == 0){
+		dh.dump_compress = DUMP_COMPRESS_GZIP;
+		compress_fn = compress_gzip;
+	}
+	else{
+		dh.dump_compress = DUMP_COMPRESS_NONE;
+		compress_fn = compress_none;
+	}
+
 	if(g.parm_dump_mem < dh.memory_size){
 		/* dump_mem parameter specified: Adjust memory size */
 		dh.memory_size = g.parm_dump_mem;
@@ -1245,7 +1277,8 @@ dump_create_s390sa(char* sourcedev, char* dumpdir)
 		PRINT_ERR("lseek() failed\n");
 		rc = -1; goto out;
 	}
-        if (dump_write(fp_dump, (char *)dump_page_buf, DUMP_BUFFER_SIZE) != DUMP_BUFFER_SIZE) {
+        if (dump_write(fp_dump, (char *)dump_page_buf, DUMP_BUFFER_SIZE)
+		!= DUMP_BUFFER_SIZE) {
 		PRINT_ERR("Error: Write dump header failed\n");
 		rc = -1; goto out;
         }
@@ -1269,7 +1302,8 @@ dump_create_s390sa(char* sourcedev, char* dumpdir)
 		/* get the new compressed page size
                  */
 
-                size = compress_fn((char *)buf, DUMP_PAGE_SIZE, (char *)dpcpage, DUMP_PAGE_SIZE);
+                size = compress_fn((unsigned char *)buf, DUMP_PAGE_SIZE,
+			(unsigned char *)dpcpage, DUMP_PAGE_SIZE);
 
                 /* if compression failed or compressed was ineffective,
                  * we write an uncompressed page
@@ -1284,18 +1318,18 @@ dump_create_s390sa(char* sourcedev, char* dumpdir)
 	        dp.address = mem_loc;
                 dp.size    = dp_size;
                 dp.flags   = dp_flags;
-                memcpy((void *)(dump_page_buf + buf_loc), (const void *)&dp, sizeof(dump_page_t));
+                memcpy((void *)(dump_page_buf + buf_loc), (const void *)&dp,
+			sizeof(dump_page_t));
                 buf_loc += sizeof(dump_page_t);
-		/* copy the page of memory
-                 */
+		/* copy the page of memory */
                 if (dp_flags & DUMP_DH_COMPRESSED) {
-                        /* copy the compressed page
-                         */
-                        memcpy((void *)(dump_page_buf + buf_loc), (const void *)dpcpage, dp_size);
+                        /* copy the compressed page */
+                        memcpy((void *)(dump_page_buf + buf_loc),
+				(const void *)dpcpage, dp_size);
                 } else {
-                        /* copy directly from memory
-                         */
-                        memcpy((void *)(dump_page_buf + buf_loc), (const void *)buf, dp_size);
+                        /* copy directly from memory */
+                        memcpy((void *)(dump_page_buf + buf_loc),
+				(const void *)buf, dp_size);
                 }
 		buf_loc += dp_size;
 		if(dump_write(fp_dump, dump_page_buf, buf_loc) != buf_loc){
@@ -1307,8 +1341,7 @@ dump_create_s390sa(char* sourcedev, char* dumpdir)
 		dump_display_progress(mem_loc, dh.memory_size);
 	}
 
-        /* write end marker 
-         */
+        /* write end marker */
  
         dp.address = 0x0;
         dp.size    = DUMP_DH_END;
