@@ -137,7 +137,6 @@ struct boot_tape_ipl_params {
 
 int boot_check_data(void);
 int boot_init_fba_stage0(struct boot_fba_stage0* stage0,
-			 disk_blockptr_t* program_table,
 			 disk_blockptr_t* stage2_list,
 			 blocknum_t stage2_count, struct disk_info* info);
 int boot_get_fba_stage2(void** data, size_t* size, struct job_data* job);
@@ -162,5 +161,6 @@ int boot_init_eckd_classic_dump_stage1(struct boot_eckd_classic_stage1* stage1,
 int boot_init_eckd_compatible_dump_stage1(
 				struct boot_eckd_compatible_stage1* stage1,
 				struct disk_info* info);
+int boot_get_fba_dump_stage2(void** data, size_t* size, uint64_t mem);
 
 #endif /* BOOT_H */
