@@ -30,6 +30,7 @@
 
 #define KERNEL_HEADER_SIZE		65536
 #define BOOTMAP_FILENAME		"bootmap"
+#define BOOTMAP_TEMPLATE_FILENAME	"bootmap_temp.XXXXXX"
 
 #define ZIPL_CONF_VAR			"ZIPLCONF"
 #define ZIPL_DEFAULT_CONF		"/etc/zipl.conf"
@@ -48,7 +49,12 @@ typedef uint64_t address_t;
 #define VOID_ADD(ptr, offset)	((void *) (((unsigned long) ptr) + \
 				((unsigned long) offset)))
 
+/* Call a function depending on the value of dry_run and return either the
+ * resulting return code or 0. */
+#define	DRY_RUN_FUNC(x)	(dry_run ? 0 : (x))
+
 extern int verbose;
 extern int interactive;
+extern int dry_run;
 
 #endif /* not ZIPL_H */
