@@ -290,12 +290,18 @@ create_component_header(void* buffer, component_header_type type)
 }
 
 
+struct component_loc {
+	address_t addr;
+	size_t size;
+};
+
 static int
 add_component_file(int fd, const char* filename, address_t load_address,
 		   off_t offset, void* component, int add_files,
-		   struct disk_info* info)
+		   struct disk_info* info, struct component_loc *location)
 {
 	struct disk_info* file_info;
+	struct component_loc loc;
 	disk_blockptr_t segment;
 	disk_blockptr_t* list;
 	char* buffer;
@@ -338,9 +344,10 @@ add_component_file(int fd, const char* filename, address_t load_address,
 			error_reason("File is not on target device");
 			return -1;
 		}
-		disk_free_info(file_info);
 		/* Get block list from existing file */
-		count = disk_get_blocklist_from_file(filename, &list, info);
+		count = disk_get_blocklist_from_file(filename, &list,
+						     file_info);
+		disk_free_info(file_info);
 		if (count == 0)
 			return -1;
 		if (count * info->phy_block_size <= (size_t) offset) {
@@ -358,6 +365,9 @@ add_component_file(int fd, const char* filename, address_t load_address,
 				list[to] = list[from];
 		}
 	}
+	/* Fill in component location */
+	loc.addr = load_address;
+	loc.size = count * info->phy_block_size;
 	/* Try to compact list */
 	count = disk_compact_blocklist(list, count, info);
 	/* Write segment table */
@@ -366,6 +376,9 @@ add_component_file(int fd, const char* filename, address_t load_address,
 	if (rc == 0) {
 		create_component_entry(component, &segment, component_load,
 				       load_address, info);
+		/* Return location if requested */
+		if (location != NULL)
+			*location = loc;
 	}
 	return rc;
 }
@@ -373,8 +386,10 @@ add_component_file(int fd, const char* filename, address_t load_address,
 
 static int
 add_component_buffer(int fd, void* buffer, size_t size, address_t load_address,
-		     void* component, struct disk_info* info)
+		     void* component, struct disk_info* info,
+		     struct component_loc *location)
 {
+	struct component_loc loc;
 	disk_blockptr_t segment;
 	disk_blockptr_t* list;
 	blocknum_t count;
@@ -386,6 +401,9 @@ add_component_buffer(int fd, void* buffer, size_t size, address_t load_address,
 		error_text("Could not write to bootmap file");
 		return -1;
 	}
+	/* Fill in component location */
+	loc.addr = load_address;
+	loc.size = count * info->phy_block_size;
 	/* Try to compact list */
 	count = disk_compact_blocklist(list, count, info);
 	/* Write segment table */
@@ -394,11 +412,63 @@ add_component_buffer(int fd, void* buffer, size_t size, address_t load_address,
 	if (rc == 0) {
 		create_component_entry(component, &segment, component_load,
 				       load_address, info);
+		/* Return location if requested */
+		if (location != NULL)
+			*location = loc;
 	}
 	return rc;
 }
 
 
+static int
+is_in_component(address_t a, struct component_loc *loc)
+{
+	return (a >= loc->addr && a < (loc->addr + loc->size));
+}
+
+
+static int
+check_component_overlap(const char *name[], struct component_loc *loc, int num)
+{
+	const char *padding = "................";
+	int overlap;
+	int i;
+	int j;
+
+	if (verbose)
+		printf("  component address:\n");
+	overlap = -1;
+	/* Process all available components */
+	for (i = 0; i < num; i++) {
+		if (loc[i].size == 0)
+			continue;
+		for (j = 0; j < num; j++) {
+			if (loc[j].size == 0 || i == j)
+				continue;
+			/* Check for overlap */
+			if (is_in_component(loc[i].addr, &loc[j]) ||
+			    is_in_component(loc[j].addr, &loc[i])) {
+				/* Set error text only for first occurrence */
+				if (overlap == -1) {
+					error_reason("%s and %s", name[i],
+						     name[j]);
+				}
+				overlap = i;
+			}
+			
+		}
+		if (verbose)
+			printf("    %s%s: 0x%08llx-0x%08llx%s\n", name[i],
+			       &padding[strlen(name[i])],
+			       (unsigned long long) loc[i].addr,
+			       (unsigned long long) (loc[i].addr +
+						     loc[i].size - 1),
+			       (i == overlap ? " (overlap)" : ""));
+	}
+	return overlap != -1;
+}
+
+
 static int
 add_ipl_program(int fd, struct job_ipl_data* ipl, disk_blockptr_t* program,
 		int verbose, int add_files, component_header_type type,
@@ -408,9 +478,13 @@ add_ipl_program(int fd, struct job_ipl_data* ipl, disk_blockptr_t* program,
 	void* table;
 	void* stage3;
 	size_t stage3_size;
+	const char *comp_name[4] = {"kernel image", "parmline",
+				    "initial ramdisk", "internal loader"};
+	struct component_loc comp_loc[4];
 	int rc;
 	int offset;
 
+	memset(comp_loc, 0, sizeof(comp_loc));
 	table = misc_malloc(info->phy_block_size);
 	if (table == NULL)
 		return -1;
@@ -422,12 +496,11 @@ add_ipl_program(int fd, struct job_ipl_data* ipl, disk_blockptr_t* program,
 	offset += sizeof(struct component_header);
 	/* Add kernel image */
 	if (verbose) {
-		printf("  kernel image......: %s at 0x%llx\n", ipl->image,
-		       (unsigned long long) ipl->image_addr);
+		printf("  kernel image......: %s\n", ipl->image);
 	}
 	rc = add_component_file(fd, ipl->image, ipl->image_addr,
 				KERNEL_HEADER_SIZE, VOID_ADD(table, offset),
-				add_files, info);
+				add_files, info, &comp_loc[0]);
 	if (rc) {
 		error_text("Could not add image file '%s'", ipl->image);
 		free(table);
@@ -437,15 +510,13 @@ add_ipl_program(int fd, struct job_ipl_data* ipl, disk_blockptr_t* program,
 	if (ipl->parmline != NULL) {
 		/* Add kernel parmline */
 		if (verbose) {
-			printf("  kernel parmline...: '%s' at 0x%llx\n",
-			       ipl->parmline,
-			       (unsigned long long) ipl->parm_addr);
+			printf("  kernel parmline...: '%s'\n", ipl->parmline);
 		}
 		rc = add_component_buffer(fd, ipl->parmline,
 					  strlen(ipl->parmline) + 1,
 					  ipl->parm_addr,
 					  VOID_ADD(table, offset),
-					  info);
+					  info, &comp_loc[1]);
 		if (rc) {
 			error_text("Could not add parmline '%s'",
 				   ipl->parmline);
@@ -458,9 +529,7 @@ add_ipl_program(int fd, struct job_ipl_data* ipl, disk_blockptr_t* program,
 	if (ipl->ramdisk != NULL) {
 		/* Add ramdisk */
 		if (verbose) {
-			printf("  initial ramdisk...: %s at 0x%llx\n",
-			       ipl->ramdisk,
-			       (unsigned long long) ipl->ramdisk_addr);
+			printf("  initial ramdisk...: %s\n", ipl->ramdisk);
 		}
 		/* Get ramdisk file size */
 		if (stat(ipl->ramdisk, &stats)) {
@@ -473,7 +542,7 @@ add_ipl_program(int fd, struct job_ipl_data* ipl, disk_blockptr_t* program,
 		rc = add_component_file(fd, ipl->ramdisk,
 					ipl->ramdisk_addr, 0,
 					VOID_ADD(table, offset),
-					add_files, info);
+					add_files, info, &comp_loc[2]);
 		if (rc) {
 			error_text("Could not add ramdisk '%s'",
 				   ipl->ramdisk);
@@ -493,7 +562,7 @@ add_ipl_program(int fd, struct job_ipl_data* ipl, disk_blockptr_t* program,
 	}
 	rc = add_component_buffer(fd, stage3, stage3_size,
 				  DEFAULT_STAGE3_ADDRESS,
-				  VOID_ADD(table, offset), info);
+				  VOID_ADD(table, offset), info, &comp_loc[3]);
 	free(stage3);
 	if (rc) {
 		error_text("Could not add stage 3 boot loader");
@@ -501,6 +570,14 @@ add_ipl_program(int fd, struct job_ipl_data* ipl, disk_blockptr_t* program,
 		return -1;
 	}
 	offset += sizeof(struct component_entry);
+	/* Check for component overlaps */
+	rc = check_component_overlap(comp_name, comp_loc, 4);
+	if (rc)
+	{
+		error_text("Component overlap");
+		free(table);
+		return -1;
+	}
 	/* Terminate component table */
 	create_component_entry(VOID_ADD(table, offset), NULL,
 			       component_execute,
@@ -519,10 +596,13 @@ add_segment_program(int fd, struct job_segment_data* segment,
 		    disk_blockptr_t* program, int verbose, int add_files,
 		    component_header_type type, struct disk_info* info)
 {
+	const char *comp_name[1] = {"segment file"};
+	struct component_loc comp_loc[1];
 	void* table;
 	int offset;
 	int rc;
 
+	memset(comp_loc, 0, sizeof(comp_loc));
 	table = misc_malloc(info->phy_block_size);
 	if (table == NULL)
 		return -1;
@@ -534,12 +614,11 @@ add_segment_program(int fd, struct job_segment_data* segment,
 	offset += sizeof(struct component_header);
 	/* Add segment file */
 	if (verbose) {
-		printf("  segment file......: %s at 0x%llx\n",
-		       segment->segment,
-		       (unsigned long long) segment->segment_addr);
+		printf("  segment file......: %s\n", segment->segment);
 	}
 	rc = add_component_file(fd, segment->segment, segment->segment_addr, 0,
-				VOID_ADD(table, offset), add_files, info);
+				VOID_ADD(table, offset), add_files, info,
+				&comp_loc[0]);
 	if (rc) {
 		error_text("Could not add segment file '%s'",
 			   segment->segment);
@@ -547,6 +626,8 @@ add_segment_program(int fd, struct job_segment_data* segment,
 		return rc;
 	}
 	offset += sizeof(struct component_entry);
+	/* Print component addresses */
+	check_component_overlap(comp_name, comp_loc, 1);
 	/* Terminate component table */
 	create_component_entry(VOID_ADD(table, offset), NULL,
 			       component_execute, PSW_DISABLED_WAIT, info);
@@ -794,17 +875,18 @@ bootmap_create(struct job_data* job, disk_blockptr_t* program_table,
 	struct disk_info* info;
 	char* device;
 	char* filename;
+	char *mapname;
 	void* stage2_data;
 	size_t stage2_size;
 	int fd;
 	int rc;
 
 	/* Get full path of bootmap file */
-	filename = misc_make_path(job->bootmap_dir, BOOTMAP_FILENAME);
+	filename = misc_make_path(job->bootmap_dir, BOOTMAP_TEMPLATE_FILENAME);
 	if (filename == NULL)
 		return -1;
-	/* Create bootmap file */
-	fd = open(filename, O_RDWR | O_TRUNC | O_CREAT, S_IRUSR | S_IWUSR);
+	/* Create temporary bootmap file */
+	fd = mkstemp(filename);
 	if (fd == -1) {
 		error_reason(strerror(errno));
 		error_text("Could not create file '%s':", filename);
@@ -858,7 +940,7 @@ bootmap_create(struct job_data* job, disk_blockptr_t* program_table,
 			return rc;
 		}
 	}
-	printf("Building bootmap '%s'%s\n", filename,
+	printf("Building bootmap in '%s'%s\n", job->bootmap_dir,
 	       job->add_files ? " (files will be added to bootmap file)" :
 	       "");
 	/* Write bootmap header */
@@ -939,9 +1021,34 @@ bootmap_create(struct job_data* job, disk_blockptr_t* program_table,
 		*stage2_count = 0;
 		break;
 	}
+	close(fd);
+	if (dry_run) {
+		if (remove(filename) == -1)
+			fprintf(stderr, "Warning: could not remove temporary "
+				"file %s!\n", filename);
+	} else {
+		/* Rename to final bootmap name */
+		mapname = misc_make_path(job->bootmap_dir, BOOTMAP_FILENAME);
+		if (mapname == NULL) {
+			misc_free_temp_dev(device);
+			disk_free_info(info);
+			free(filename);
+			return -1;
+		}
+		rc = rename(filename, mapname);
+		if (rc) {
+			error_reason(strerror(errno));
+			error_text("Could not overwrite file '%s':", mapname);
+			misc_free_temp_dev(device);
+			disk_free_info(info);
+			free(mapname);
+			free(filename);
+			return rc;
+		}
+		free(mapname);
+	}
 	*new_device = device;
 	*new_info = info;
-	close(fd);
 	free(filename);
 	return rc;
 }
