@@ -34,9 +34,7 @@
 /* from linux/hdregs.h */
 #define HDIO_GETGEO		0x0301
 
-#define DASD_MIN_API_VERSION	0
 #define DASD_IOCTL_LETTER	'D'
-#define DASDAPIVER		_IOR(DASD_IOCTL_LETTER, 0, int)
 #define BIODASDINFO		_IOR(DASD_IOCTL_LETTER, 1, \
 				     struct dasd_information)
 #define DASD_PARTN_MASK		0x03
@@ -72,8 +70,6 @@ disk_get_info(const char* device, struct disk_info** info)
 	struct dasd_information dasd_info;
 	struct disk_info *data;
 	int fd;
-	int rc;
-	int dasd_api;
 	long devsize;
 
 	/* Get file information */
@@ -87,22 +83,6 @@ disk_get_info(const char* device, struct disk_info** info)
 		error_reason(strerror(errno));
 		return -1;
 	}
-	/* Check DASD driver API version */
-	rc = ioctl(fd, DASDAPIVER, &dasd_api);
-	if (rc == -1)
-		dasd_api = -1;
-	else if (rc) {
-		error_reason("Unexpected driver response (%d)", rc);
-		close(fd);
-		return -1;
-	}
-	/* Check API version */
-	if ((dasd_api < DASD_MIN_API_VERSION) && (dasd_api != -1)) {
-		error_reason("Unsupported DASD driver API version (%d)",
-			     dasd_api);
-		close(fd);
-		return -1;
-	}
 	/* Get memory for result */
 	data = (struct disk_info *) misc_malloc(sizeof(struct disk_info));
 	if (data == NULL) {
@@ -118,20 +98,14 @@ disk_get_info(const char* device, struct disk_info** info)
 		close(fd);
 		return -1;
 	}
-	/* Get DASD information in case of DASD device */
-	if (dasd_api != -1) {
-		if (ioctl(fd, BIODASDINFO, &dasd_info)) {
-			error_reason("Could not get information from DASD "
-				     "driver");
-			free(data);
-			close(fd);
-			return -1;
-		}
-		data->devno = dasd_info.devno;
-	} else
+	/* Get DASD information */
+	if (ioctl(fd, BIODASDINFO, &dasd_info))
 		data->devno = -1;
+	else
+		data->devno = dasd_info.devno;
+
 	/* Determine disk type */
-	if (dasd_api == -1)
+	if (data->devno == -1)
 		/* FIXME: SCSI check? */
 		data->type = disk_type_scsi;
 	else if (strncmp(dasd_info.type, "FBA ",4) == 0)
@@ -466,11 +440,11 @@ disk_get_type_name(disk_type_t type)
 {
 	switch (type) {
 	case disk_type_scsi:
-		return "SCSI";
+		return "SCSI disk layout";
 	case disk_type_fba:
-		return "FBA";
+		return "FBA disk layout";
 	case disk_type_diag:
-		return "DIAG";
+		return "DIAG disk layout";
 	case disk_type_eckd_classic:
 		return "ECKD/linux disk layout";
 	case disk_type_eckd_compatible:
