@@ -134,7 +134,7 @@ install_scsi(int fd, disk_blockptr_t* program_table, struct disk_info* info)
 		free(bootblock);
 		return -1;
 	}
-	rc = misc_write(fd, bootblock, info->phy_block_size);
+	rc = DRY_RUN_FUNC(misc_write(fd, bootblock, info->phy_block_size));
 	if (rc)
 		error_text("Could not write master boot record");
 	free(bootblock);
@@ -152,11 +152,13 @@ install_fba(int fd, disk_blockptr_t* program_table,
 	int rc;
 
 	/* Initialize stage 0 data */
-	rc = boot_init_fba_stage0(&stage0, program_table, stage2_list,
-				  stage2_count, info);
+	rc = boot_init_fba_stage0(&stage0, stage2_list, stage2_count, info);
 	if (rc == 0) {
+		/* Store program table pointer in stage 0 loader */
+		bootmap_store_blockptr(&stage0.param1, program_table, info);
 		/* Install stage 0 */
-		rc = misc_write(fd, &stage0, sizeof(struct boot_fba_stage0));
+		rc = DRY_RUN_FUNC(misc_write(
+			fd, &stage0, sizeof(struct boot_fba_stage0)));
 	}
 	if (rc)
 		error_text("Could not write boot loader");
@@ -178,7 +180,8 @@ install_eckd_classic(int fd, disk_blockptr_t* program_table,
 	/* Prepare stage 0 */
 	boot_init_eckd_classic_stage0(&stage0);
 	/* Install stage 0 */
-	rc = misc_write(fd, &stage0, sizeof(struct boot_eckd_stage0));
+	rc = DRY_RUN_FUNC(misc_write(fd, &stage0,
+		sizeof(struct boot_eckd_stage0)));
 	if (rc) {
 		error_text("Could not write boot loader");
 		return rc;
@@ -197,7 +200,8 @@ install_eckd_classic(int fd, disk_blockptr_t* program_table,
 		error_text("Could not seek on device");
 		return -1;
 	}
-	rc = misc_write(fd, &stage1, sizeof(struct boot_eckd_classic_stage1));
+	rc = DRY_RUN_FUNC(misc_write(fd, &stage1,
+		sizeof(struct boot_eckd_classic_stage1)));
 	if (rc) {
 		error_text("Could not write boot loader");
 		return rc;
@@ -227,7 +231,8 @@ install_eckd_compatible(int fd, disk_blockptr_t* program_table,
 		error_text("Could not seek on device");
 		return -1;
 	}
-	rc = misc_write(fd, &stage0, sizeof(struct boot_eckd_stage0));
+	rc = DRY_RUN_FUNC(misc_write(fd, &stage0,
+		sizeof(struct boot_eckd_stage0)));
 	if (rc) {
 		error_text("Could not write boot loader");
 		return -1;
@@ -245,8 +250,8 @@ install_eckd_compatible(int fd, disk_blockptr_t* program_table,
 		error_text("Could not seek on device");
 		return -1;
 	}
-	rc = misc_write(fd, &stage1,
-			sizeof(struct boot_eckd_compatible_stage1));
+	rc = DRY_RUN_FUNC(misc_write(fd, &stage1,
+		sizeof(struct boot_eckd_compatible_stage1)));
 	if (rc) {
 		error_text("Could not write boot loader");
 		return -1;
@@ -261,7 +266,7 @@ install_eckd_compatible(int fd, disk_blockptr_t* program_table,
 	rc = boot_get_eckd_stage2(&buffer, &size, job);
 	if (rc)
 		return -1;
-	rc = misc_write(fd, buffer, size);
+	rc = DRY_RUN_FUNC(misc_write(fd, buffer, size));
 	if (rc)
 		error_text("Could not write boot loader");
 	free(buffer);
@@ -319,7 +324,7 @@ install_bootloader(const char* device, disk_blockptr_t *program_table,
 		break;
 	}
 	close(fd);
-	if (rc == 0) {
+	if (!dry_run && rc == 0) {
 		if (info->devno >= 0)
 			syslog(LOG_INFO, "Boot loader written to %s (%04x) - "
 			       "%02x:%02x",
@@ -517,14 +522,15 @@ install_tapeloader(const char* device, const char* image, const char* parmline,
 		return -1;
 	}
 	/* Write boot loader */
-	rc = write_tapebuffer(fd, buffer, size, IPL_TAPE_BLOCKSIZE);
+	rc = DRY_RUN_FUNC(write_tapebuffer(fd, buffer, size,
+		IPL_TAPE_BLOCKSIZE));
 	free(buffer);
 	if (rc) {
 		error_text("Could not write boot loader to tape");
 		close(fd);
 		return rc;
 	}
-	rc = write_tapemark(fd, 1);
+	rc = DRY_RUN_FUNC(write_tapemark(fd, 1));
 	if (rc) {
 		error_text("Could not write boot loader to tape");
 		close(fd);
@@ -535,13 +541,13 @@ install_tapeloader(const char* device, const char* image, const char* parmline,
 		printf("  kernel image......: %s at 0x%llx\n", image,
 		       (unsigned long long) image_addr);
 	}
-	rc = write_tapefile(fd, image, IPL_TAPE_BLOCKSIZE);
+	rc = DRY_RUN_FUNC(write_tapefile(fd, image, IPL_TAPE_BLOCKSIZE));
 	if (rc) {
 		error_text("Could not write image file '%s' to tape", image);
 		close(fd);
 		return rc;
 	}
-	rc = write_tapemark(fd, 1);
+	rc = DRY_RUN_FUNC(write_tapemark(fd, 1));
 	if (rc) {
 		error_text("Could not write boot loader to tape");
 		close(fd);
@@ -553,15 +559,15 @@ install_tapeloader(const char* device, const char* image, const char* parmline,
 			       parmline, (unsigned long long) parm_addr);
 		}
 		/* Write parameter line */
-		rc = write_tapebuffer(fd, parmline, strlen(parmline),
-				      IPL_TAPE_BLOCKSIZE);
+		rc = DRY_RUN_FUNC(write_tapebuffer(fd, parmline,
+			strlen(parmline), IPL_TAPE_BLOCKSIZE));
 		if (rc) {
 			error_text("Could not write parameter string to tape");
 			close(fd);
 			return rc;
 		}
 	}
-	rc = write_tapemark(fd, 1);
+	rc = DRY_RUN_FUNC(write_tapemark(fd, 1));
 	if (rc) {
 		error_text("Could not write boot loader to tape");
 		close(fd);
@@ -573,7 +579,8 @@ install_tapeloader(const char* device, const char* image, const char* parmline,
 			printf("  initial ramdisk...: %s at 0x%llx\n",
 			       ramdisk, (unsigned long long) initrd_addr);
 		}
-		rc = write_tapefile(fd, ramdisk, IPL_TAPE_BLOCKSIZE);
+		rc = DRY_RUN_FUNC(write_tapefile(fd, ramdisk,
+			IPL_TAPE_BLOCKSIZE));
 		if (rc) {
 			error_text("Could not write ramdisk file '%s' to tape",
 				   ramdisk);
@@ -581,7 +588,7 @@ install_tapeloader(const char* device, const char* image, const char* parmline,
 			return rc;
 		}
 	}
-	rc = write_tapemark(fd, 1);
+	rc = DRY_RUN_FUNC(write_tapemark(fd, 1));
 	if (rc) {
 		error_text("Could not write boot loader to tape");
 		close(fd);
@@ -617,7 +624,8 @@ install_dump_eckd_classic(int fd, struct disk_info* info, uint64_t mem)
 	/* Prepare stage 0 */
 	boot_init_eckd_classic_stage0(&stage0);
 	/* Install stage 0 */
-	rc = misc_write(fd, &stage0, sizeof(struct boot_eckd_stage0));
+	rc = DRY_RUN_FUNC(misc_write(fd, &stage0,
+		sizeof(struct boot_eckd_stage0)));
 	if (rc) {
 		error_text("Could not write boot loader");
 		return rc;
@@ -649,7 +657,8 @@ install_dump_eckd_classic(int fd, struct disk_info* info, uint64_t mem)
 		error_text("Could not seek on device");
 		return -1;
 	}
-	rc = misc_write(fd, &stage1, sizeof(struct boot_eckd_classic_stage1));
+	rc = DRY_RUN_FUNC(misc_write(fd, &stage1,
+		sizeof(struct boot_eckd_classic_stage1)));
 	if (rc) {
 		error_text("Could not write dump record");
 		return rc;
@@ -663,7 +672,7 @@ install_dump_eckd_classic(int fd, struct disk_info* info, uint64_t mem)
 	rc = boot_get_eckd_dump_stage2(&buffer, &size, mem);
 	if (rc)
 		return rc;
-	rc = misc_write(fd, buffer, size);
+	rc = DRY_RUN_FUNC(misc_write(fd, buffer, size));
 	if (rc)
 		error_text("Could not write dump record");
 	free(buffer);
@@ -689,7 +698,8 @@ install_dump_eckd_compatible(int fd, struct disk_info* info, uint64_t mem)
 		error_text("Could not seek on device");
 		return -1;
 	}
-	rc = misc_write(fd, &stage0, sizeof(struct boot_eckd_stage0));
+	rc = DRY_RUN_FUNC(misc_write(fd, &stage0,
+		sizeof(struct boot_eckd_stage0)));
 	if (rc) {
 		error_text("Could not write boot loader");
 		return rc;
@@ -722,8 +732,8 @@ install_dump_eckd_compatible(int fd, struct disk_info* info, uint64_t mem)
 		error_text("Could not seek on device");
 		return -1;
 	}
-	rc = misc_write(fd, &stage1,
-			sizeof(struct boot_eckd_compatible_stage1));
+	rc = DRY_RUN_FUNC(misc_write(fd, &stage1,
+			sizeof(struct boot_eckd_compatible_stage1)));
 	if (rc) {
 		error_text("Could not write dump record");
 		return rc;
@@ -737,7 +747,7 @@ install_dump_eckd_compatible(int fd, struct disk_info* info, uint64_t mem)
 	rc = boot_get_eckd_dump_stage2(&buffer, &size, mem);
 	if (rc)
 		return rc;
-	rc = misc_write(fd, buffer, size);
+	rc = DRY_RUN_FUNC(misc_write(fd, buffer, size));
 	if (rc)
 		error_text("Could not write dump record");
 	free(buffer);
@@ -745,6 +755,68 @@ install_dump_eckd_compatible(int fd, struct disk_info* info, uint64_t mem)
 }
 
 
+static int
+install_dump_fba(int fd, struct disk_info* info, uint64_t mem)
+{
+	struct boot_fba_stage0 stage0;
+	disk_blockptr_t stage2_list[8];
+	blocknum_t block;
+	blocknum_t count;
+	void* buffer;
+	size_t size;
+	int rc;
+	int i;
+
+	/* Install stage 2 dump record at end of partition */
+	rc = boot_get_fba_dump_stage2(&buffer, &size, mem);
+	if (rc)
+		return rc;
+	count = (size + info->phy_block_size - 1) / info->phy_block_size;
+	if (count > 8) {
+		error_reason("FBA dump record is too large");
+		free(buffer);
+		return -1;
+	}
+	block = info->geo.start + info->phy_blocks - count;
+	if (lseek(fd, block * info->phy_block_size, SEEK_SET) !=
+	    block * info->phy_block_size) {
+		error_reason(strerror(errno));
+		error_text("Could not seek on device");
+		free(buffer);
+		return -1;
+	}
+	rc = DRY_RUN_FUNC(misc_write(fd, buffer, size));
+	free(buffer);
+	if (rc) {
+		error_text("Could not write to device");
+		return rc;
+	}
+	for (i=0; i < count; i++) {
+		stage2_list[i].linear.block = block + i;
+		stage2_list[i].linear.size = info->phy_block_size;
+		stage2_list[i].linear.blockct = 0;
+	}
+	/* Prepare stage 0 */
+	rc = boot_init_fba_stage0(&stage0, stage2_list, count, info);
+	if (rc)
+		return rc;
+	/* Fill in dump parameters */
+	stage0.param1 = (uint64_t) info->geo.start;
+	stage0.param2 = (uint64_t) block - 1;
+	/* Install stage 0 */
+	if (lseek(fd, 0, SEEK_SET) != 0) {
+		error_reason(strerror(errno));
+		error_text("Could not seek on device");
+		return -1;
+	}
+	rc = DRY_RUN_FUNC(misc_write(fd, &stage0,
+		sizeof(struct boot_fba_stage0)));
+	if (rc)
+		error_text("Could not write dump record");
+	return rc;
+}
+
+
 static int
 install_dump_tape(int fd, uint64_t mem)
 {
@@ -755,7 +827,7 @@ install_dump_tape(int fd, uint64_t mem)
 	rc = boot_get_tape_dump(&buffer, &size, mem);
 	if (rc)
 		return rc;
-	rc = misc_write(fd, buffer, size);
+	rc = DRY_RUN_FUNC(misc_write(fd, buffer, size));
 	if (rc)
 		error_text("Could not write to tape device");
 	free(buffer);
@@ -781,7 +853,7 @@ install_dump(const char* device, uint64_t mem)
 		/* Rewind worked - this is a tape */
 		rc = ask_for_confirmation("Warning: All information on device "
 					  "'%s' will be lost!\nDo you want to "
-					  "continue creating a one-shot dump "
+					  "continue creating a dump "
  					  "tape (y/n) ?", device);
 		if (rc) {
 			close(fd);
@@ -837,6 +909,7 @@ install_dump(const char* device, uint64_t mem)
 	switch (info->type) {
 	case disk_type_eckd_classic:
 	case disk_type_eckd_compatible:
+	case disk_type_fba:
 		rc = ask_for_confirmation("Warning: All information on "
 					  "partition '%s' will be lost!\n"
 					  "Do you want to continue creating "
@@ -849,11 +922,12 @@ install_dump(const char* device, uint64_t mem)
 		}
 		if (info->type == disk_type_eckd_classic)
 			rc = install_dump_eckd_classic(fd, info, mem);
-		else
+		else if (info->type == disk_type_eckd_compatible)
 			rc = install_dump_eckd_compatible(fd, info, mem);
+		else
+			rc = install_dump_fba(fd, info, mem);
 		break;
 	case disk_type_scsi:
-	case disk_type_fba:
 	case disk_type_diag:
 	case disk_type_unknown:
 		error_reason("Unsupported disk type '%s' of dump target '%s'",
