@@ -43,6 +43,7 @@ static struct option options[] = {
 	{ "verbose",		no_argument,		NULL, 'V'},
 	{ "add-files",		no_argument,		NULL, 'a'},
 	{ "tape",		required_argument,	NULL, 'T'},
+	{ "dry-run",		no_argument,		NULL, '0'},
 	{ NULL,			0,			NULL, 0 }
 };
 
@@ -59,6 +60,7 @@ struct command_line {
 	int version;
 	int verbose;
 	int add_files;
+	int dry_run;
 	enum scan_section_type type;
 };
 
@@ -171,6 +173,9 @@ get_command_line(int argc, char* argv[], struct command_line* line)
 		case 'a':
 			cmdline.add_files = 1;
 			break;
+		case '0':
+			cmdline.dry_run = 1;
+			break;
 		case 1:
 			/* Non-option is interpreted as section name */
 			if (cmdline.section != NULL) {
@@ -1211,6 +1216,14 @@ get_job_from_config_file(struct command_line* cmdline, struct job_data* job)
 		rc = get_section_job(scan, cmdline->section, job,
 				cmdline->data[(int) scan_keyword_parameters]);
 	}
+	/* Make sure no '--parameters' option was specified when writing a 
+	 * menu section. */
+	if (job->id == job_menu &&
+	    cmdline->data[(int) scan_keyword_parameters]) {
+		error_text("Option 'parameters' cannot be used with a menu "
+			   "section");
+		rc = -1;
+	}
 	scan_free(scan);
 	return rc;
 }
@@ -1234,6 +1247,7 @@ job_get(int argc, char* argv[], struct job_data** data)
 	job->noninteractive = cmdline.noninteractive;
 	job->verbose = cmdline.verbose;
 	job->add_files = cmdline.add_files;
+	job->dry_run = cmdline.dry_run;
 	/* Get job data from user input */
 	if (cmdline.help) {
 		job->command_line = 1;
