@@ -717,7 +717,7 @@ scan_check_section_data(char* keyword[], int* line, char* name,
 		    (keyword[(int) scan_keyword_defaultmenu] == 0)) {
 			error_reason("Line %d: Section 'defaultboot' requires "
 				     "one of keywords 'default' and "
-				     "'defaultboot'", section_line);
+				     "'defaultmenu'", section_line);
 			return -1;
 		}
 		
