@@ -15,6 +15,8 @@
 #include <string.h>
 #include <errno.h>
 #include <unistd.h>
+#include <sys/types.h>
+#include <sys/stat.h>
 
 #include "boot.h"
 #include "bootmap.h"
@@ -31,6 +33,9 @@ int verbose = 0;
 /* Flag deciding whether confirmation questions are asked */
 int interactive = 1;
 
+/* Flag deciding whether actions should only be simulated */
+int dry_run = 1;
+
 /* Full tool name */
 static const char tool_name[] = "zSeries Initial Program Loader";
 
@@ -68,6 +73,7 @@ static const char* usage_text[] = {
 "-n, --noninteractive            Answer all confirmation questions with 'yes'",
 "-V, --verbose                   Provide more verbose output",
 "-a, --add-files                 Add all referenced files to bootmap file",
+"    --dry-run                   Simulate run but don't modify IPL records",
 ""
 };
 
@@ -148,6 +154,12 @@ main(int argc, char* argv[])
 	/* Set global option variables */
 	verbose = job->verbose;
 	interactive = !job->noninteractive;
+	dry_run = job->dry_run;
+	if (dry_run)
+		printf("Starting dry-run, target device contents will NOT be "
+		       "modified\n");
+	/* Make sure new files are only user-accessible */
+	umask(077);
 	/* Do it */
 	switch (job->id) {
 	case job_ipl:
@@ -191,7 +203,8 @@ main(int argc, char* argv[])
 	else {
 		if (verbose)
 			printf("Syncing disks...\n");
-		sync();
+		if (!dry_run)
+			sync();
 		printf("Done.\n");
 	}
 	job_free(job);
